/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */
/************************************************************************
 *
 * File Name:  App.c
 * Description: GUI main control module, which open/close handles to
 *              the SyncClient engine, and manage user preferences
 *
 ************************************************************************
 */

#include "app.h"
#include "logfile.h"
#include "messages.h"

#ifdef UNDER_PO
#include "SyncForm.h"
#include "StarterRsc.h"
#endif //UNDER_PO

#if defined(UNDER_PO) || defined(UNDER_CE) || defined (UNDER_NT)
APPDATA app;
#endif

#ifndef UNDER_PO
int PrefGetAppPreferences(isy_TCHAR* creator, int id, AppPrefs* prefsPtr, isy_UINT16* prefsSizePtr, int flag)
{
	isy_TCHAR fileName[100];
	HISYNCFILE file;

	TSTR_CPY(fileName, DATADIR);
	TSTR_CAT(fileName, creator);
	TSTR_CAT(fileName, isy_T("prefs"));
	
	file = FOPEN(fileName, isy_T("rb"));

	if (file == NULL)
		return -1;

	// If reading old preferences, this new field will not be set
	prefsPtr->messageSize = appDefaultMessageSize;		// default to none, e.g. 0

	FREAD(prefsPtr, sizeof(AppPrefs), 1,  file);
	FCLOSE(file);

	return 0;
}


void PrefSetAppPreferences(isy_TCHAR* creator, int id, int version, AppPrefs* prefsPtr, 
							long size, int flag)
{
	isy_TCHAR fileName[50];
	HISYNCFILE file;
	//GET_APP_

	TSTR_CPY(fileName, DATADIR);
	TSTR_CAT(fileName, creator);
	TSTR_CAT(fileName, isy_T("prefs"));
	file = FOPEN(fileName, isy_T("wb+"));
	FWRITE(prefsPtr, size, 1, file);
	FCLOSE(file);
}
#endif	//not UNDER_PO

int appReadPreferences()
{
	int rc;
	isy_UINT16 prefsSize = sizeof(AppPrefs);
	GET_APP_

	int i=0, iPseudoKey = 721, len = 0;

	//PrefGetAppPreferences is a built-in Palm function
	//EPOC and other platforms must redefine this...
	rc = PrefGetAppPreferences(appFileCreator, appPrefID, &(_pApp->prefs), &prefsSize, true);

	// decrypt appPrefPassword(pApp)
	if (appPrefVersion(_pApp) >= ISYNC_VersionNum810 && appPrefPassword(_pApp)[0])
	{
		len = TSTR_LEN(appPrefPassword(_pApp));
		for (i=0; i<len; i++) appPrefPassword(_pApp)[i] ^= iPseudoKey;
	}

	return rc;
}

int appSavePreferences()
{
	int rc = 0;
	int i=0, iPseudoKey = 721, len = 0;
	isy_TCHAR origPassword[ISYNCLEN_Password];
	GET_APP_

	// Encrypt password
	TSTR_CPY(origPassword, appPrefPassword(_pApp));
	if (appPrefVersion(_pApp) >= ISYNC_VersionNum810 && appPrefPassword(_pApp)[0])
	{
		len = TSTR_LEN(appPrefPassword(_pApp));
		for (i=0; i<len; i++) appPrefPassword(_pApp)[i] ^= iPseudoKey;
	}

	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&(_pApp->prefs), sizeof(_pApp->prefs), true);

	TSTR_CPY(appPrefPassword(_pApp), origPassword);
	return rc;
}

isy_INT appGetPref(isy_INT prefID, isy_TCHAR *prefVal, const isy_INT prefLen)
{
	isy_INT		rc;
	int		  	prefSize = 0;
	isy_TCHAR 	timeoutBuf[ISYNCLEN_TimeoutRecv];
	GET_APP_

	rc = ISCRTN_Succeeded;
	switch(prefID) 
	{
		case ISYNCPREF_URL:
			TSTR_N_CPY(prefVal, appPrefURL(_pApp), prefLen);
			prefVal[prefLen - 1] = 0;
			prefSize = TSTR_LEN(appPrefURL(_pApp));
			break;

		case ISYNCPREF_User:
			TSTR_N_CPY(prefVal, appPrefUser(_pApp), prefLen);
			prefVal[prefLen - 1] = 0;
			prefSize = TSTR_LEN(appPrefUser(_pApp));
			break;

		case ISYNCPREF_UnsavedPassword:
			if (appPrefOptionSavePwd(_pApp)) {
				rc = ISCRTN_True; // password is actually saved
			}
		case ISYNCPREF_Password:
			TSTR_N_CPY(prefVal, appPrefPassword(_pApp), prefLen);
			prefVal[prefLen - 1] = 0;
			prefSize = TSTR_LEN(appPrefPassword(_pApp));
			break;

		case ISYNCPREF_TimeoutSend:
		case ISYNCPREF_TimeoutRecv:
			TSTR_ITOA(timeoutBuf, (prefID == ISYNCPREF_TimeoutSend)
				? appPrefSendTimeout(_pApp) : appPrefRecvTimeout(_pApp));
			prefSize = TSTR_LEN(timeoutBuf);
			TSTR_N_CPY(prefVal, timeoutBuf, (prefSize >= prefLen ? prefLen : prefSize+1));
			prefVal[prefLen - 1] = 0;
			break;

		case ISYNCPREF_Message_Size:
			TSTR_ITOA(timeoutBuf, appPrefMessageSize(_pApp));
			prefSize = TSTR_LEN(timeoutBuf);
			TSTR_N_CPY(prefVal, timeoutBuf, (prefSize >= prefLen ? prefLen : prefSize+1));
			prefVal[prefLen - 1] = 0;
			break;

		case ISYNCPREF_Trace:
			prefSize = 1;
			prefVal[0] = appPrefOptionTrace(_pApp) ? (isy_TCHAR) 'Y' : (isy_TCHAR) 'N';
			prefVal[1] = 0;
			break;

		case ISYNCPREF_TargetDir:
			TSTR_N_CPY(prefVal, appPrefTargetDir(_pApp), prefLen);
			prefVal[prefLen - 1] = 0;
			prefSize = TSTR_LEN(appPrefTargetDir(_pApp));
			break;

		case ISYNCPREF_TargetSlot:
			prefSize = 2;
			TSTR_N_CPY(prefVal, appPrefTargetSlot(_pApp), prefLen);
			prefVal[prefLen - 1] = 0;
			prefSize = TSTR_LEN(appPrefTargetSlot(_pApp));
			break;

		case ISYNCPREF_Proxy:
			TSTR_N_CPY(prefVal, appPrefProxy(_pApp), prefLen);
			prefVal[prefLen - 1] = 0;
			prefSize = TSTR_LEN(appPrefProxy(_pApp));
			break;

		default:
			rc = ISCRTN_UnknownID;
			break;
	}

	if (rc == ISCRTN_Succeeded && prefLen <= prefSize) 
	{
		rc = ISCRTN_ValTruncated;
	}
	return rc;
}

isy_INT appSetPref(isy_INT prefID, const isy_TCHAR *prefVal)
{
	isy_INT   rc = ISCRTN_Succeeded;
	int      prefSize = 0;
	GET_APP_

	if (prefVal == NULL) return ISCRTN_Failed;

	switch(prefID) {
		case ISYNCPREF_URL:
			// Set the old dest field
			prefSize = sizeof(appPrefDest(_pApp))/sizeof(isy_TCHAR);
			TSTR_N_CPY(appPrefDest(_pApp), prefVal, prefSize);
			appPrefDest(_pApp)[prefSize - 1] = 0;
			// Set the extended URL field
			prefSize = sizeof(appPrefURL(_pApp))/sizeof(isy_TCHAR);
			TSTR_N_CPY(appPrefURL(_pApp), prefVal, prefSize);
			appPrefURL(_pApp)[prefSize - 1] = 0;
			break;

		case ISYNCPREF_User:
			prefSize = sizeof(appPrefUser(_pApp))/sizeof(isy_TCHAR);
			TSTR_N_CPY(appPrefUser(_pApp), prefVal, prefSize);
			appPrefUser(_pApp)[prefSize - 1] = 0;
			break;

		case ISYNCPREF_Password:
		case ISYNCPREF_UnsavedPassword:
			prefSize = sizeof(appPrefPassword(_pApp))/sizeof(isy_TCHAR);
			TSTR_N_CPY(appPrefPassword(_pApp), prefVal, prefSize);
			appPrefPassword(_pApp)[prefSize - 1] = 0;

			if (prefID == ISYNCPREF_Password)
				appSetPrefOptionSavePwd(_pApp);
			else
				appResetPrefOptionSavePwd(_pApp);
			break;

		case ISYNCPREF_TimeoutSend:
			prefSize = ISYNCLEN_TimeoutSend;
			appPrefSendTimeout(_pApp) = TSTR_ATOI(prefVal);
			break;

		case ISYNCPREF_TimeoutRecv:
			prefSize = ISYNCLEN_TimeoutRecv;
			appPrefRecvTimeout(_pApp) = TSTR_ATOI(prefVal);
			break;

		case ISYNCPREF_Message_Size:
			prefSize = ISYNCLEN_TimeoutSend;
			appPrefMessageSize(_pApp) = TSTR_ATOI(prefVal);
			break;

		case ISYNCPREF_Trace:
			prefSize = 2; // including the null char
			if (*prefVal == 'Y' || *prefVal == 'y')
				appSetPrefOptionTrace(_pApp);
			else if (*prefVal == 'N' || *prefVal == 'n')
				appResetPrefOptionTrace(_pApp);
			break;
		case ISYNCPREF_TargetDir:
			prefSize = sizeof(appPrefTargetDir(_pApp))/sizeof(isy_TCHAR);
			TSTR_N_CPY(appPrefTargetDir(_pApp), prefVal, prefSize);
			_pApp->prefs.targetDir[prefSize-1] = 0;
			break;
		case ISYNCPREF_TargetSlot:
			prefSize = 2; // including the null char
			TSTR_N_CPY(appPrefTargetSlot(_pApp), prefVal, prefSize);
			_pApp->prefs.targetSlot[prefSize-1] = 0;
			break;
		case ISYNCPREF_Proxy:
			prefSize = sizeof(appPrefProxy(_pApp))/sizeof(isy_TCHAR);
			TSTR_N_CPY(appPrefProxy(_pApp), prefVal, prefSize);
			_pApp->prefs.proxy[prefSize-1] = 0;
			break;

		default:
			rc = ISCRTN_UnknownID;
			break;
	}

	if (rc == ISCRTN_Succeeded && prefSize <= (int) TSTR_LEN(prefVal))
		rc = ISCRTN_ValTooLong;

	if (rc == ISCRTN_Succeeded) {
		appSavePreferences();
    }

	return rc;
}

void syncShutdown()
{
	GET_APP_
	
	if (_pApp->hSyncer != NULL) {
	   iscEngineClose(_pApp->hSyncer);
       _pApp->hSyncer = NULL;
	}

	if (_pApp->hSyncConf != NULL) {
	   iscConfigClose(_pApp->hSyncConf);
	   _pApp->hSyncConf = NULL;
	}

	if (_pApp->hSyncServ != NULL) {
	   iscServiceClose(_pApp->hSyncServ);
	   _pApp->hSyncServ = NULL;
	}
}

int syncStartup(iscEngineListenerPF pSyncListener) 
{
	ISCPROPERTY properties[] = {{ISCPROP_SyncUser,        NULL},
	                            {ISCPROP_SyncPassword,    NULL},
	                            {ISCPROP_SyncMessageSize, NULL},
	                            {ISCPROP_SyncTrace,       NULL},
	                            {ISCPROP_SyncTimeout,     NULL},
	                            {ISCPROP_SyncProxy,       NULL}
	};  
	int       numProperty = 0;

	isy_TCHAR URL[ISYNCLEN_URL];
	isy_TCHAR messageSize[ISYNCLEN_MessageSize];
	isy_TCHAR timeout[ISYNCLEN_TimeoutSend];	
	isy_TCHAR targetDir[ISYNCLEN_TargetDir+5];  // 5 extra for Palm slot "#1:/.../
	int       targetDirLen = 0;
	int       rc = 0;
	GET_APP_

	syncShutdown();

	appGetPref(ISYNCPREF_URL, URL, ISYNCLEN_URL);

	TSTR_ITOA(messageSize, appPrefMessageSize(_pApp));
	TSTR_ITOA(timeout, appPrefSendTimeout(_pApp));
	
	properties[numProperty++].value = appPrefUser(_pApp);
	properties[numProperty++].value = appPrefPassword(_pApp);
	properties[numProperty++].value = messageSize;
	properties[numProperty++].value = appPrefOptionTrace(_pApp) ? isy_T("detailed") : isy_T("default");
	properties[numProperty++].value = timeout;
	if (appPrefOptionProxy(_pApp))
		properties[numProperty++].value = appPrefProxy(_pApp);
	
	// rc = iscServiceOpen(server, port, appPrefUser(_pApp), appPrefPassword(_pApp), NULL, &(_pApp->hSyncServ));
	rc = iscServiceOpenEx(URL, properties, numProperty, &(_pApp->hSyncServ));
	if (rc != ISCRTN_Succeeded) return rc;

	// compose the target directory string
	targetDir[0] = (isy_TCHAR) '\0';
#ifdef UNDER_PO
	if (TSTR_CMP(appPrefTargetSlot(_pApp), isy_T("0")) != 0) {
		int targetSlot = TSTR_ATOI(appPrefTargetSlot(_pApp)); // assuming > 0

		TSTR_CAT(targetDir, isy_T("#"));
		TSTR_ITOA(targetDir+TSTR_LEN(targetDir), targetSlot-1);
		TSTR_CAT(targetDir, isy_T(":"));
		if (*appPrefTargetDir(_pApp) != (isy_TCHAR) '/')
			TSTR_CAT(targetDir, isy_T("/")); // Always an absolute path

		TSTR_CAT(targetDir, appPrefTargetDir(_pApp));
		targetDirLen = TSTR_LEN(targetDir);			
	}
#else
		TSTR_CAT(targetDir, appPrefTargetDir(_pApp));
		targetDirLen = TSTR_LEN(targetDir);			
#endif
	

	if (targetDirLen > 0 && targetDir[targetDirLen - 1] != (isy_TCHAR) FPATH_SLASH)
		TSTR_CAT(targetDir, isy_T(FPATH_SLASHSTR)); // DB2e requires path ended with slash

	rc = iscConfigOpen(_pApp->hSyncServ, targetDir, &(_pApp->hSyncConf));
	if (rc != ISCRTN_Succeeded) return rc;

	rc = iscEngineOpen(_pApp->hSyncConf, &(_pApp->hSyncer));
	if (rc != ISCRTN_Succeeded) return rc;


	iscEngineSetListener(_pApp->hSyncer, pSyncListener, (isy_UINT32) NULL);

	return rc;
}

//d7281 hchiu
#ifdef UNDER_CE
static int getV8B2AppPref(isy_TCHAR* fileName, int id, isy_BYTE *p, isy_UINT16* pPrefSize, int flag)
{
	HISYNCFILE  file = FOPEN(fileName, isy_T("rb"));

	if (file == NULL)
		return noPreferenceFound;

	MEM_SET(p, 0, *pPrefSize);
	*pPrefSize = FREAD(p, *pPrefSize, 1, file);
	FCLOSE(file);

	return 0;
}
#endif

void appInitialize()
{
	GET_APP_

	_pApp->hLog = logOpen(_pApp);
	MEM_SET(&_pApp->prefs, 0, sizeof(AppPrefs));
	if (appReadPreferences() == noPreferenceFound)
	{ 
		// Read rushmore 4.1 (DB2e v8 beta2) preferences
#ifdef UNDER_CE
		V8B2AppPrefs v8b2prefs;
		isy_UINT16 prefSize = sizeof(V8B2AppPrefs);
		isy_TCHAR fileName[128];
		int rc=0, i=0, len=0, iPseudoKey = 721;

		//TSTR_CPY(fileName, ISYNCDIR);
		TSTR_CPY(fileName, DATADIR);
		TSTR_CAT(fileName, appFileCreator);
		TSTR_CAT(fileName, isy_T(".PROPERTIES"));
		
		rc = getV8B2AppPref(fileName, 0, (isy_BYTE *) &v8b2prefs, &prefSize, true);
		if (rc != noPreferenceFound)
		{
			appPrefVersion(_pApp) = v8b2prefs.version.num;
			appPrefSize(_pApp)    = sizeof(AppPrefs);
			appPrefSendTimeout(_pApp) = v8b2prefs.timeoutSend;
			appPrefRecvTimeout(_pApp) = v8b2prefs.timeoutReceive;
			
			TSTR_CPY(appPrefUser(_pApp), v8b2prefs.user);
			TSTR_CPY(appPrefPassword(_pApp), v8b2prefs.password);
			//decrypt
			len = TSTR_LEN(appPrefPassword(_pApp));
			for (i=0; i<len; i++) appPrefPassword(_pApp)[i] ^= iPseudoKey;

			TSTR_CPY(appPrefURL(_pApp), prefOptionSSL(v8b2prefs) ? isy_T("https://") : isy_T("http://"));
			TSTR_CAT(appPrefURL(_pApp), v8b2prefs.dest);
			TSTR_CPY(appPrefDest(_pApp), appPrefURL(_pApp));
			//TSTR_CPY(appPrefTargetDir(_pApp), v8b2prefs.targetDir);
			TSTR_CPY(appPrefTargetDir(_pApp), DATADIR);
									
			// trace option
			appResetPrefOptions(_pApp);
			if (v8b2prefs.options & ISYNCOPTION_SavePassword) appSetPrefOptionSavePwd(_pApp);
			if (v8b2prefs.options & ISYNCOPTION_Trace) appSetPrefOptionTrace(_pApp);

			//delete old SERVER-ISYN (v8b2 server list)
			TSTR_CPY(fileName, DATADIR);
			TSTR_CAT(fileName, isy_T("server-isyn"));
			REMOVE(fileName);
		}
		else
		{
#endif //UNDER_CE

		// give initial settings when no previous preferences exist
		appPrefVersion(_pApp) = iscGetVersion();
		appPrefSize(_pApp)    = sizeof(AppPrefs);
		appPrefSendTimeout(_pApp) = appDefaultTimeout * 60; // seconds
		appPrefRecvTimeout(_pApp) = appDefaultTimeout * 60; // seconds
		appPrefMessageSize(_pApp) = 0;
		
		TSTR_CPY(appPrefUser(_pApp),isy_T(""));
		TSTR_CPY(appPrefPassword(_pApp),isy_T(""));
		TSTR_CPY(appPrefURL(_pApp), appDefaultURL);
		TSTR_CPY(appPrefDest(_pApp), appDefaultURL);
		TSTR_CPY(appPrefTargetDir(_pApp), DATADIR);
		TSTR_CPY(appPrefTargetSlot(_pApp), isy_T("0"));
		
		// trace option
		appResetPrefOptions(_pApp);
		if (appDefaultSavePassword)
			appSetPrefOptionSavePwd(_pApp);

		if (appDefaultDetailedLog) 
			appSetPrefOptionTrace(_pApp);

#ifdef UNDER_CE
		}
#endif //UNDER_CE
		appSavePreferences(); // save the initial settings

	}
	else
	{ 
		// previous preferences read
		// in case not initialized previously - to prevent crash
		appPrefUser(_pApp)[sizeof(appPrefUser(_pApp))/sizeof(isy_TCHAR)-1] = 0;
		appPrefPassword(_pApp)[sizeof(appPrefPassword(_pApp))/sizeof(isy_TCHAR)-1] = 0;
		if (! appPrefOptionSavePwd(_pApp) && *appPrefPassword(_pApp) != '\0') 
		{
			MEM_SET(appPrefPassword(_pApp), 0, sizeof(appPrefPassword(_pApp)));
		}
		if (TSTR_LEN(appPrefURL(_pApp)) == 0) {
			isy_TCHAR *protocol = TSTR_STR(appPrefDest(_pApp), isy_T("://"));
			if (protocol == NULL) {
				// make http as the default transport protocol
				TSTR_CPY(appPrefURL(_pApp), appPrefOptionSSL(_pApp) ? isy_T("https://") : isy_T("http://"));
				TSTR_CAT(appPrefURL(_pApp), appPrefDest(_pApp));
				TSTR_N_CPY(appPrefDest(_pApp), appPrefURL(_pApp), 50);
				appPrefDest(_pApp)[49] = (isy_TCHAR) '\0';
			} else {
				TSTR_CPY(appPrefURL(_pApp), appPrefDest(_pApp));
			}
			appResetPrefOptionSSL(_pApp); // Not relying on PrefOptionSSL any more
		}
	}
	
	_pApp->canceled = false;

#ifdef UNDER_PO	
	_pApp->mute = (PrefGetPreference(prefSysSoundVolume) == 0);
#endif 	
}

void appTerminate()
{
	GET_APP_
	syncShutdown();	
	if (_pApp->hLog) {
		logClose(_pApp->hLog);
		_pApp->hLog = NULL;
	}
}

isy_BOOL appMessageBox(const isy_TCHAR* pMsg, int iType)
{
	return (isy_BOOL) appMessageBox2(pMsg, iType, -1, false);
}

// return 1 if OK/Yes pressed, 0 if cancel/no pressed, retTimeout if timeout
int appMessageBox2(const isy_TCHAR* pMsg, int iType, long timeout, int retTimeout)
{
#if defined(UNDER_PO)
	UInt u;
	Word w;	

	if (iType == APPMB_YESNO)
		u = EXTMB_YESNO;
	else if (iType == APPMB_OKCANCEL)
		u = EXTMB_OK | EXTMB_CANCEL;
	else
		u = EXTMB_OK;

	// 9999 could be anything that's different from Message*Button
	w = extMessageBoxPtr((isy_TCHAR*)pMsg, u, timeout, 9999);
	if (w == MessageOKButton || w == MessageYesButton)
		return 1;
	else if (w == 9999)
		return retTimeout;
	else 
		return 0;
	// UNDER_PO

#elif defined(UNDER_EP)
	EPOCGLOBALS* globals = epocGetGlobals();
	EPUIDATA* dataPtr = (EPUIDATA*) globals->hData;
	int w;
	//unsigned int u;

	if (iType == 0)
		iType = APPMB_OK;

	w = dataPtr->messageBox((isy_TCHAR*)pMsg, dataPtr->engine, iType, NULL);
	return w == MessageOKButton;
	// UNDER_EP

#else
	return 0;
#endif
}

const isy_TCHAR* appEventCodeToMessage(isy_INT32 eventCode)
{	
	switch (eventCode) 
	{
	case ISCEVT_InfSyncStarted:
		return ISYNCSTR_InfSyncStarted;
	case ISCEVT_InfPrepMsg:
		return ISYNCSTR_InfPrepMsg;
	case ISCEVT_InfSendMsg:
		return ISYNCSTR_InfSendMsg;
	case ISCEVT_InfWaitMsg:
		return ISYNCSTR_InfWaitMsg;
	case ISCEVT_InfApplyMsg:
		return ISYNCSTR_InfApplyMsg;
	case ISCEVT_InfCancelingSync:
		return ISYNCSTR_InfCancelingSync;
	case ISCEVT_InfSubsSetStarted:
		return ISYNCSTR_InfSubsSetStarted;
	case ISCEVT_InfSyncingSubs:
		return ISYNCSTR_InfSyncingSubs;
	case ISCEVT_InfSubsSetFailed:
		return ISYNCSTR_InfSubsSetFailed;
	case ISCEVT_InfSubsSetCanceled:
		return ISYNCSTR_InfSubsSetCanceled;
	case ISCEVT_InfSubsSetSucceeded:
		return ISYNCSTR_InfSubsSetSucceeded;
	case ISCEVT_InfSyncSucceeded:
		return ISYNCSTR_InfSyncSucceeded;
	case ISCEVT_InfSyncFailed:
		return ISYNCSTR_InfSyncFailed;
	case ISCEVT_InfSyncCanceled:
		return ISYNCSTR_InfSyncCanceled;
	case ISCEVT_InfSyncProg:
	case ISCEVT_InfSyncMsgSent:
		return ISYNCSTR_InfSyncProg;
	case ISCEVT_InfNoNewChange:
		return ISYNCSTR_InfNoNewChange;

	// Conflict events
	case ISCEVT_CftReject:
		return ISYNCSTR_CftReject;

	// Error events
	case ISCEVT_ErrOpenAdapter:
		return ISYNCSTR_ErrOpenAdapter;
	case ISCEVT_ErrLoadAdapter:
		return ISYNCSTR_ErrLoadAdapter;
	case ISCEVT_ErrCloseAdapter:
		return ISYNCSTR_ErrCloseAdapter;
	case ISCEVT_ErrAuthenticateKey:
		return ISYNCSTR_ErrAuthenticateKey;
	case ISCEVT_ErrClientCryptoFailed:
		return ISYNCSTR_ErrClientCryptoFailed;
	case ISCEVT_ErrEncryptNotAvail:
		return ISYNCSTR_ErrEncryptNotAvail;
	case ISCEVT_ErrEncryptLibOpen:
		return ISYNCSTR_ErrEncryptLibOpen;
	case ISCEVT_ErrSubsNotFound:
		return ISYNCSTR_ErrSubsNotFound;
	case ISCEVT_ErrSubsNotAvail:
		return ISYNCSTR_ErrSubsNotAvail;
	case ISCEVT_ErrSubsDefAltered:
		return ISYNCSTR_ErrSubsDefAltered;
	
	case ISCEVT_ErrCloseNetLib:
		return ISYNCSTR_ErrCloseNetLib;
	case ISCEVT_ErrOutOfMemory:
		return ISYNCSTR_ErrOutOfMemory;

	case ISCEVT_ErrAllocResource:
		return ISYNCSTR_ErrAllocResource;
	case ISCEVT_ErrConnectData:
		return ISYNCSTR_ErrConnectData;
	case ISCEVT_ErrDisconnectData:
		return ISYNCSTR_ErrDisconnectData;
	case ISCEVT_ErrNoData:
		return ISYNCSTR_ErrNoData;
	case ISCEVT_ErrSyncDisabled:
		return ISYNCSTR_ErrSyncDisabled;
	case ISCEVT_ErrServerException:
		return ISYNCSTR_ErrServerException;
	case ISCEVT_ErrMessageFormat:
		return ISYNCSTR_ErrMessageFormat;
	case ISCEVT_ErrNotFound:
		return ISYNCSTR_ErrNotFound;
	case ISCEVT_ErrEndOfData:
		return ISYNCSTR_ErrEndOfData;
	case ISCEVT_ErrDataTooLong:
		return ISYNCSTR_ErrDataTooLong;
	case ISCEVT_ErrReadOnly:
		return ISYNCSTR_ErrReadOnly;
	case ISCEVT_ErrOperation:
		return ISYNCSTR_ErrOperation;
	case ISCEVT_ErrUnauthorized:
		return ISYNCSTR_ErrUnauthorized;
	case ISCEVT_ErrNotAvailable:
		return ISYNCSTR_ErrNotAvailable;
	case ISCEVT_ErrNotSupported:
		return ISYNCSTR_ErrNotSupported;

	case ISCEVT_ErrInternal:
		return ISYNCSTR_ErrInternal;

	// Fatal events
	case ISCEVT_FatSyncCfgAbort:
		return ISYNCSTR_FatSyncCfgAbort;
	case ISCEVT_FatAuthenticateFailed:
		return ISYNCSTR_FatAuthenticateFailed;
	case ISCEVT_FatIncompVersion:
		return ISYNCSTR_FatIncompVersion;
	case ISCEVT_FatInvalidSession:
		return ISYNCSTR_FatInvalidSession;
	case ISCEVT_FatSyncGroup:
		return ISYNCSTR_FatSyncGroup;
	case ISCEVT_FatRegisterDevice:
		return ISYNCSTR_FatRegisterDevice;
	case ISCEVT_FatServerForbidden:
		return ISYNCSTR_FatServerForbidden;
	case ISCEVT_FatServerNotFound:
		return ISYNCSTR_FatServerNotFound;
	case ISCEVT_FatServerNotAvail:
		return ISYNCSTR_FatServerNotAvail;
	case ISCEVT_FatServer:
		return ISYNCSTR_FatServer;
	case ISCEVT_FatProtocolNotSupported:
		return ISYNCSTR_FatProtocolNotSupported;
	case ISCEVT_FatServerBusy:
		return ISYNCSTR_FatServerBusy;
	case ISCEVT_FatNetOpenConn:
		return ISYNCSTR_FatNetOpenConn;
	case ISCEVT_FatOpenNetLib:
		return ISYNCSTR_FatOpenNetLib;
	case ISCEVT_FatNetConnect:
		return ISYNCSTR_FatNetConnect;
	case ISCEVT_FatNetSend:
		return ISYNCSTR_FatNetSend;
	case ISCEVT_FatNetReceive:
		return ISYNCSTR_FatNetReceive;
	case ISCEVT_FatNetTimeout:
		return ISYNCSTR_FatNetTimeout;
	case ISCEVT_FatResolveHost:
		return ISYNCSTR_FatResolveHost;
	case ISCEVT_FatCreateImage:
		return ISYNCSTR_FatCreateImage;
	case ISCEVT_FatNetUnknown:
		return ISYNCSTR_FatNetUnknown;

	// Retry events
	case ISCEVT_TryServerBusy:
		return ISYNCSTR_TryServerBusy; 
	case ISCEVT_TryNetConnect:
		return ISYNCSTR_TryNetConnect; 
	case ISCEVT_TryNetSend:
		return ISYNCSTR_TryNetSend;
	case ISCEVT_TryNetReceive:
		return ISYNCSTR_TryNetReceive;
	case ISCEVT_TryNetTimeout:
		return ISYNCSTR_TryNetTimeout ;
	
	// Query events
	case ISCEVT_QueCancel:
		return ISYNCSTR_QueCancel;
	case ISCEVT_QueLogin:
		return ISYNCSTR_QueLogin;
	case ISCEVT_QueCancelUponError:
		return ISYNCSTR_QueCancelUponError;
	case ISCEVT_QueSubsTarget:
		return ISYNCSTR_QueSubsTarget;
	
	default:
		return isy_T("");
	}

	return NULL;
} // appEventToMessage

const isy_TCHAR* appCurrentDateTime(isy_TCHAR* buf, isy_TCHAR* formatString)
{
	const isy_TCHAR* psz;
	int y, m, d, hr, mi, se;
	int i, k;
	
	psz = ISYNCSTR_DateTime;
	
	OSGET_CURRENT_DATE_TIME(&y, &m, &d, &hr, &mi, &se);
	for (i = 0, k = 0; psz[i]; i++)
	{
		switch (psz[i]) {
		case 'y':
			TSTR_ITOA(buf + k, y);
			k = TSTR_LEN(buf);
			break;
		case 'm':
			TSTR_ITOA(buf + k, m);
			k = TSTR_LEN(buf);
			break;
		case 'd':
			TSTR_ITOA(buf + k, d);
			k = TSTR_LEN(buf);
			break;
		case 'h':
			if (hr < 10) buf[k++] = '0';
			TSTR_ITOA(buf + k, hr);
			k = TSTR_LEN(buf);
			break;
		case 't':
			if (mi < 10) buf[k++] = '0';
			TSTR_ITOA(buf + k, mi);
			k = TSTR_LEN(buf);
			break;
		case 's':
			if (se < 10) buf[k++] = '0';
			TSTR_ITOA(buf + k, se);
			k = TSTR_LEN(buf);
			break;
		default:
			buf[k] = psz[i];
			++k;
		}
	}
	buf[k] = 0;
	
	return buf;
}

void appLogWrite(APPDATA *pApp, isy_INT32 eventCode, isy_INT32 eventType, const isy_TCHAR* msg)
{
	isy_TCHAR buf[LOG_LINE_LEN];

	MEM_SET(buf, 0, LOG_LINE_LEN * sizeof(isy_TCHAR));
	// Get the timestamp
	appCurrentDateTime(buf, (isy_TCHAR*) ISYNCSTR_DateTime);
	TSTR_CAT(buf, isy_T(" DSYC"));

	if (eventCode / 1000 == 0) TSTR_CAT(buf, isy_T("0"));
	TSTR_ITOA(buf+TSTR_LEN(buf), eventCode);
	switch (eventType) {
	  case ISCEVTTYPE_Info  :
		TSTR_CAT(buf, isy_T("I: "));
		break;
	  case ISCEVTTYPE_Conflict  :
		TSTR_CAT(buf, isy_T("C: "));
		break;
	  case ISCEVTTYPE_Query     :
		TSTR_CAT(buf, isy_T("Q: "));
		break;
	  case ISCEVTTYPE_Retry	    :
		TSTR_CAT(buf, isy_T("R: "));
		break;
	  case ISCEVTTYPE_Error     :
	  case ISCEVTTYPE_Fatal     :
		TSTR_CAT(buf, isy_T("E: "));
		break;
	  default:
		TSTR_CAT(buf, isy_T("?: "));
		break;
	}
	TSTR_N_CAT(buf, msg, LOG_LINE_LEN - TSTR_LEN(buf) - 1);
	logWrite(pApp->hLog, buf);
} // appLogWrite

void appLogBuildVersion(APPDATA *pApp)
{
    isy_TCHAR verMsg[128];

	TSTR_CPY(verMsg, isy_T("DB2 Everyplace V"));
	appVersionToString(pApp, verMsg + TSTR_LEN(verMsg));
	TSTR_CAT(verMsg, isy_T(" "));
	
	appLogWrite(pApp, ISCEVT_InfGeneral, ISCEVTTYPE_Info, verMsg);
} // appLogBuildVersion

void appVersionToString(APPDATA *pApp, isy_TCHAR *verString)
{
    isy_UINT32  version;
    int         verMajor, verMinor;
	int         verMod, verFix;

	version = iscGetVersion();

	verMajor = (int) (version >> 24);
	verMinor = (int) ((version >> 16) & 0x000000FF);
	verMod   = (int) ((version >> 8) & 0x000000FF);
	verFix   = (int) ((version) & 0x000000FF);

	TSTR_ITOA(verString, verMajor);
	TSTR_CAT(verString, isy_T("."));
	TSTR_ITOA(verString + TSTR_LEN(verString), verMinor);
	TSTR_CAT(verString, isy_T("."));
	TSTR_ITOA(verString + TSTR_LEN(verString), verMod);
	TSTR_CAT(verString, isy_T("."));
	TSTR_ITOA(verString + TSTR_LEN(verString), verFix);
} // appVersionToString
