/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */
/************************************************************************
 *
 * File Name:  LogForm.c
 * Description: Handle log keeping and displaying
 *
 ************************************************************************
 */

#ifdef UNDER_PO_31
#	include <Pilot.h>
#	define WinDirectionType DirectionType
#	define winUp            up
#	define winDown          down
#else
#	include <PalmOS.h>
#	include <PalmCompatibility.h>
#endif
#include <SysEvtMgr.h>
#include "StarterRsc.h"

#include "app.h"
#include "MainForm.h"
#include "LogForm.h"

static void pageScroll(FormPtr frmP, WinDirectionType direction);
static void scroll(FormPtr frmP, Short linesToScroll);
static void updateScrollBar(FormPtr frmP);

// cbString excludes terminating 0
// append NO terminating 0
// cbDataCurrent excludes terminating 0
static VoidHand appendString(VoidHand hMem, unsigned long cbDataCurrent, CharPtr pString, unsigned long cbString)
{
	unsigned long bufSize = MemHandleSize(hMem);
	CharPtr pc;
	if (cbDataCurrent + cbString + 1 > bufSize)
	{
		unsigned long incr = 100;
		int n = (cbDataCurrent + cbString + 1) / 100;
		bufSize = (n + 1) * incr;
		MemHandleResize(hMem, bufSize);
	}

	pc = (CharPtr) MemHandleLock(hMem);
	MemMove(pc + cbDataCurrent, pString, cbString);
	MemHandleUnlock(hMem);
	return hMem;
}

static void formInit(FormPtr frmP)
{
	UInt cRecords;
	VoidHand hRecord;
	CharPtr pc;
	UInt cb;
	UInt cbData;
	UInt i, startRec;
	FieldPtr fld = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, LogEditField));
	VoidHand h = MemHandleNew(500);
	VoidHand hOld;
	DmOpenRef dbref;
	GET_APP_
	
	dbref = DmOpenDatabaseByTypeCreator(appLogDbType, appFileCreator, dmModeReadOnly);
	if (dbref == 0) return;
	
	cRecords = DmNumRecordsInCategory(dbref, dmAllCategories);
	cbData = 0;
	
	if(cRecords > 400)
		startRec = cRecords - 400;
	else
		startRec = 0;
		
	for (i = startRec; i < cRecords; i++)
	{
		hRecord = DmQueryRecord(dbref, i);
		cb = MemHandleSize(hRecord);
		pc = MemHandleLock(hRecord);
		
		// no terminating 0 in hRecord
		h = appendString(h, cbData, pc, cb);
		cbData += cb;

		h = appendString(h, cbData, "\n", 1);
		cbData += 1;

		MemHandleUnlock(hRecord);
	}

	h = appendString(h, cbData, "\0", 1);
	hOld = FldGetTextHandle(fld);
	FldSetTextHandle(fld, h);
	if (hOld) MemHandleFree(hOld);
	
	updateScrollBar(frmP);
	DmCloseDatabase(dbref);
	
	FrmCopyTitle(frmP, ISYNCSTR_Log);
	setCtrlLabel(frmP, LogDoneButton, ISYNCSTR_OK);
}

static Boolean formDoCommand(Word command)
{
	Boolean handled = false;

	switch (command) {
	default:
		break;
	}
	
	return handled;
}

Boolean LogFormHandleEvent(EventPtr eventP)
{
    Boolean handled = false;
    FormPtr frmP;

	switch (eventP->eType) {
	case menuEvent:
		return formDoCommand(eventP->data.menu.itemID);

	case frmOpenEvent:
		frmP = FrmGetActiveForm();
		formInit(frmP);
		FrmDrawForm(frmP);
		handled = true;
		break;
	case ctlSelectEvent:
		switch (eventP->data.ctlSelect.controlID) {
		case LogDoneButton:
			FrmGotoForm(MainForm);
			handled = true;
			break;
		}
		break;
   	case keyDownEvent:
   		if (eventP->data.keyDown.chr == pageUpChr)
		{
			pageScroll(FrmGetActiveForm(), winUp);
			handled = true;
		}
		else if (eventP->data.keyDown.chr == pageDownChr)
		{
			pageScroll(FrmGetActiveForm(), winDown);
			handled = true;
		}
		break;
	case sclRepeatEvent:
		scroll (FrmGetActiveForm(), eventP->data.sclRepeat.newValue - eventP->data.sclRepeat.value);
	   	// Repeating controls don't repeat if handled is set true.
		break;
	default:
		break;
	
	}
	
	return handled;
}

static void pageScroll(FormPtr frmP, WinDirectionType direction)
{
	Short value;
	Short min;
	Short max;
	Short pageSize;
	Word linesToScroll;
	ScrollBarPtr bar;
	FieldPtr fld = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, LogEditField));
	
	if (FldScrollable (fld, direction))
	{
		linesToScroll = FldGetVisibleLines (fld) - 1;
		FldScrollField (fld, linesToScroll, direction);

		// Update the scroll bar.
		bar = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, LogScrollScrollBar));
		SclGetScrollBar (bar, &value, &min, &max, &pageSize);

		if (direction == winUp)
			value -= linesToScroll;
		else
			value += linesToScroll;
		
		SclSetScrollBar (bar, value, min, max, pageSize);
	}
}

static void scroll(FormPtr frmP, Short linesToScroll)
{
	Word blankLines;
	Short min;
	Short max;
	Short value;
	Short pageSize;
	ScrollBarPtr bar;
	FieldPtr fld = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, LogEditField));

	if (linesToScroll < 0)
	{
		blankLines = FldGetNumberOfBlankLines (fld);
		FldScrollField (fld, -linesToScroll, winUp);
		
		// If there were blank lines visible at the end of the field
		// then we need to update the scroll bar.
		if (blankLines)
		{
			// Update the scroll bar.
			bar = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, LogScrollScrollBar));
			SclGetScrollBar (bar, &value, &min, &max, &pageSize);
			if (blankLines > -linesToScroll)
				max += linesToScroll;
			else
				max -= blankLines;
			SclSetScrollBar (bar, value, min, max, pageSize);
		}
	}
	else if (linesToScroll > 0)
		FldScrollField (fld, linesToScroll, winDown);
}

static void updateScrollBar(FormPtr frmP)
{
	Word scrollPos;
	Word textHeight;
	Word fieldHeight;
	Short maxValue;
	ScrollBarPtr bar;
	FieldPtr fld = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, LogEditField));

	bar = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, LogScrollScrollBar));
	
	// get the values necessary to update the scroll bar.
	FldGetScrollValues (fld, &scrollPos, &textHeight,  &fieldHeight);

	if (textHeight > fieldHeight)
		maxValue = textHeight - fieldHeight;
	else if (scrollPos)
		maxValue = scrollPos;
	else
		maxValue = 0;

	SclSetScrollBar (bar, scrollPos, 0, maxValue, fieldHeight-1);
}
