/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */
/************************************************************************
 *
 * File Name:  MainForm.c
 * Description: Front panel
 *
 ************************************************************************
 */

#ifdef UNDER_PO_31
#	include <Pilot.h>
#else
#	include <PalmOS.h>
#	include <PalmCompatibility.h>
#endif
#include <SysEvtMgr.h>
#include "StarterRsc.h"

#include "app.h"
#include "MainForm.h"
#include "UserForm.h"

static char g_msg1[100] = { '\0' };
static char g_msg2[100] = { '\0' };

static void bitmapTap(FormPtr frmP, RectangleType* pBounds);
static void startSynchronize();
static Boolean promptUser();

#define MainMenusSubscriptionSets	1001
#define MainMenusServerSettings		1002
#define MainMenusClientSettings		1003
#define MainMenusNetworkSettings	1004
#define MainMenusAboutIBMSync		1005

static void updateMessages(FormPtr frmP)
{
    Word w = FrmGetObjectIndex(frmP, MainMsg1Label);
    FrmHideObject(frmP, w);
    // TODO: restrict length
    FrmCopyLabel(frmP, MainMsg1Label, g_msg1);
    FrmShowObject(frmP, w);

    w = FrmGetObjectIndex(frmP, MainMsg2Label);
    FrmHideObject(frmP, w);
    // TODO: restrict length
    FrmCopyLabel(frmP, MainMsg2Label, g_msg2);
    FrmShowObject(frmP, w);
}

void setLabel(FormPtr frmP, UInt16 labelID, const Char *newLabel)
{
	Word w = FrmGetObjectIndex(frmP, labelID);
	
	FrmHideObject(frmP, w);
	FrmCopyLabel(frmP, labelID, newLabel);
	FrmShowObject(frmP, w);
}

void setCtrlLabel(FormPtr frmP, UInt16 labelID, const Char *newLabel)
{
	Word w = FrmGetObjectIndex(frmP, labelID);
	ControlType *controlP;

	controlP =	FrmGetObjectPtr(frmP, w);
	CtlSetLabel(controlP, newLabel);		
}

static void MainFormInit(FormPtr frmP)
{

    FrmHideObject(frmP, FrmGetObjectIndex(frmP, MainSettingsButton));
    FrmHideObject(frmP, FrmGetObjectIndex(frmP, MainApplicationsButton));
    
	FrmCopyTitle(frmP, ISYNCSTR_DB2Sync);
	setCtrlLabel(frmP, MainLogButton, ISYNCSTR_Log);
	setLabel(frmP, MainUnnamed1006Label, ISYNCSTR_Synchronize);
}

static void onAbout()
{
    FormPtr prevForm = FrmGetActiveForm();
    FormPtr frm;
    char    about[256];
    int rc;
    GET_APP_ // get the App handle
	
	rc = syncStartup(NULL);
	if (rc == ISCRTN_Succeeded) 
	{
		frm = FrmInitForm(AboutForm);
		
		FrmCopyTitle(frm, ISYNCSTR_About);
		setCtrlLabel(frm, AboutAboutQBEOkButton, ISYNCSTR_OK);
		
		//      if (prevForm) FrmEraseForm(prevForm);
		FrmSetActiveForm(frm);
		//      FrmSetEventHandler(frm, UserFormHandleEvent);
		FrmDrawForm(frm);

		if (iscEngineGetInfo(_pApp->hSyncer, about, 256) == ISCRTN_Succeeded) {
			FieldPtr fld = GetObjectPtr(frm, AboutSyncClientIDField);
			Handle  hOld = FldGetTextHandle(fld);
			Handle  hNew = MemHandleNew((TSTR_LEN(about) + 1) * sizeof(isy_TCHAR));
			isy_TCHAR *text = (isy_TCHAR *) MemHandleLock(hNew);

			TSTR_CPY(text, about);
			MemHandleUnlock(hNew);
			FldSetTextHandle(fld, hNew);
			FldDrawField(fld);
			if (hOld) MemHandleFree(hOld);
		}
		 FrmDoDialog(frm);
	    
	    if (prevForm)
		{
	    	FrmSetActiveForm(prevForm);
	        FrmDrawForm(prevForm);
	    }
	    FrmDeleteForm(frm);

	}
	else
	{
		isy_TCHAR statusMsg[128];
		statusMsg[0] = '\0';
		TSTR_CPY(statusMsg, ISYNCSTR_StartupError);
		TSTR_ITOA(statusMsg + TSTR_LEN(statusMsg), rc);
		appMessageBox(statusMsg, APPMB_OK);
	
	}
	syncShutdown();

	

}

static Boolean MainFormDoCommand(Word command)
{
    Boolean handled = false;

    switch (command) {
    case MainMenusServerSettings:
            doServerForm();
            handled = true;
            break;
    case MainMenusClientSettings:
            doUserForm();
            handled = true;
            break;
    case MainMenusNetworkSettings:
            doNetworkForm();
            handled = true;
            break;
    case MainMenusSubscriptionSets:
            handled = true;
            FrmGotoForm(OptionsForm);
            break;
    case MainMenusAboutIBMSync:
            onAbout();
            handled = true;
            break;
    }
    return handled;
}

Boolean MainFormHandleEvent(EventPtr eventP)
{
    Boolean handled = false;
    FormPtr frmP;
	
    GET_APP_ // get the App handle

    switch (eventP->eType) {

    case menuOpenEvent:
//MenuHideItem not working in 3.5
/*
		MenuHideItem(MainMenusSubscriptionSets);
		MenuHideItem(MainMenusServerSettings);
		MenuHideItem(MainMenusClientSettings);
		MenuHideItem(MainMenusNetworkSettings);
		MenuHideItem(MainMenusAboutIBMSync);
*/
		//add menu items


		MenuAddItem(MainMenusMenu, MainMenusSubscriptionSets, NULL, ISYNCSTR_SubsSets_Abv);
		MenuAddItem(MainMenusSubscriptionSets, MainMenusServerSettings, NULL, ISYNCSTR_Server_Settings);
		MenuAddItem(MainMenusServerSettings, MainMenusClientSettings, NULL, ISYNCSTR_Client_Settings);
		MenuAddItem(MainMenusClientSettings, MainMenusNetworkSettings, NULL, ISYNCSTR_Network_Settings);
		MenuAddItem(MainMenusNetworkSettings, MainMenusAboutIBMSync, NULL, ISYNCSTR_About);

		handled = true;
		break;

    case menuEvent:
            return MainFormDoCommand(eventP->data.menu.itemID);
    case frmOpenEvent:
        frmP = FrmGetActiveForm();
        MainFormInit( frmP);
        updateMessages(frmP);
        FrmDrawForm(frmP);
        // startSynchronize if invoked through HotSync button
        if (_pApp->hotSync) {
                startSynchronize();
                _pApp->hotSync = false;
        }        
        handled = true;
        break;
    case ctlSelectEvent:
        switch (eventP->data.ctlSelect.controlID) {
//              case MainStartButton:
//                      handled = true;
//                      // this starts synchronization:
//                      FrmGotoForm(SyncForm);
//                      break;
        case MainLogButton:
                handled = true;
                FrmGotoForm(LogForm);
                break;
        case MainSettingsButton:
                handled = true;
                doUserForm();
                break;
        case MainApplicationsButton:
                handled = true;
                FrmGotoForm(OptionsForm);
                break;
        }
        break;
    case penDownEvent:
    {
        Word bmpIndex = FrmGetObjectIndex((frmP = FrmGetActiveForm()), MainSyncBitMap);
        RectangleType bounds;
        FrmGetObjectBounds(frmP, bmpIndex, &bounds);
        if (RctPtInRectangle(eventP->screenX, eventP->screenY, &bounds))
        {
            bitmapTap(frmP, &bounds);
            handled = true;
        }
        break;
    }
    default:
        break;

    }

    return handled;
}

static void bitmapTap(FormPtr frmP, RectangleType* pBounds)
{
    Boolean penDown;
    SWord x, y;
    Boolean wasInBounds = true;
    WinInvertRectangle(pBounds, 0);
    do
    {
        Boolean nowInBounds;
        PenGetPoint(&x, &y, &penDown);
        nowInBounds = RctPtInRectangle(x, y, pBounds);
        if (nowInBounds != wasInBounds)
        {
            wasInBounds = nowInBounds;
        }
    } while (penDown);
    WinInvertRectangle(pBounds, 0);

    if (wasInBounds)
    {     	
     	startSynchronize();     	
    }
}

static void startSynchronize()
{
    ISYNCPREFS prefs;
    GET_APP_

    appGetPref(ISYNCPREF_User, prefs.user, sizeof(prefs.user));
    appGetPref(ISYNCPREF_Password, prefs.password, sizeof(prefs.password));
    if (!(prefs.user[0]))
    {
        FrmAlert(UserIDReqdAlert);
        doServerForm();
        return;
    }

    // empty session password if previously entered:
    _pApp->sessionPassword[0] = 0;
    
    // this starts synchronization:
    FrmGotoForm(SyncForm);
}

void MainFormDisplayMessages(const char* szMsg1, const char* szMsg2)
{
    FormPtr frmP = FrmGetActiveForm();

    StrNCopy(g_msg1, szMsg1 ? szMsg1 : "", sizeof(g_msg1));
    g_msg1[sizeof(g_msg1) - 1] = 0;
    StrNCopy(g_msg2, szMsg2 ? szMsg2 : "", sizeof(g_msg2));
    g_msg2[sizeof(g_msg2) - 1] = 0;

    if (FrmGetFormId(frmP) == MainForm)
            updateMessages(frmP);
}

