/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */
/************************************************************************
 *
 * File Name:  OptionsForm.c
 * Description: Subscription sets panel (for displaying and manipulating
 *              Subscription sets)
 *
 ************************************************************************
 */

#ifdef UNDER_PO_31
#	include <Pilot.h>
#else
#	include <PalmOS.h>
#	include <PalmCompatibility.h>
#endif
#include <SysEvtMgr.h>
#include "StarterRsc.h"

#include "app.h"
#include "MainForm.h"
#include "SyncForm.h"
#include "OptionsForm.h"

typedef struct {
	isy_TCHAR       oldMode; // 'N' - skip, 'R' - refresh, 'S' - synchronized
	isy_TCHAR       newMode; // 'N' - skip, 'R' - refresh, 'S' - synchronized

	isy_BOOL		oldEnabled;
	isy_BOOL		newEnabled;
	
	isy_INT16       numSubs;
	isy_TCHAR		id[ISCLEN_SubsSetID];
	isy_TCHAR		name[ISCLEN_SubsSetName];
} tJobSyncTable;

typedef struct {
	isy_INT16       numJob;
	tJobSyncTable  *jobMode;
} tSyncTable;

#define stblJobNum(pt)              ((pt)->numJob)
#define stblJobs(pt)                ((pt)->jobMode)
#define stblJobMode(pt,i)           (stblJobs(pt)[i].newMode)
#define stblJobOldMode(pt,i)        (stblJobs(pt)[i].oldMode)
#define stblJobEnabled(pt,i)        (stblJobs(pt)[i].newEnabled)
#define stblJobOldEnabled(pt,i)     (stblJobs(pt)[i].oldEnabled)

#define stblJobEnabledDirty(pt,i)   (stblJobEnabled(pt,i) != stblJobOldEnabled(pt,i))
#define stblJobModeDirty(pt,i)      (stblJobMode(pt,i) != stblJobOldMode(pt,i))

#define syncJobID(pj)               ((pj)->id)
#define syncJobName(pj)             ((pj)->name)
#define syncSubsNum(pj)             ((pj)->numSubs)

#define stblJobID(pt,i)             syncJobID(&stblJobs(pt)[i])
#define stblJobName(pt,i)           syncJobName(&stblJobs(pt)[i])
#define stblSubsNum(pt,i)           syncSubsNum(&stblJobs(pt)[i])

#define stblJobModeCached(pt,i)     (stblSubsNum(pt,i) >= 0)
#define stblJobModeReset(pt,i)      (stblSubsNum(pt,i) == -2)

#define syncModeToItem(m)		    ((m) == ISYNCMODE_Synchronize ? 1 : 2)
#define syncItemToMode(i)		    ((i) == 1 ? ISYNCMODE_Synchronize : ISYNCMODE_Refresh)

static int selectedRow = -1;

// Subscription sets ___________________________________________________
static int g_cItems = -1;      // total # of items, -1 indicates first time
static int g_indTopItem = 0;   // item # of the top viewable item
static int g_cVisibles = 0;    // not used
static long g_ridTopItem = -1; // not used

// Subscriptions _______________________________________________________
static int g_scItems = -1;     // total # of items, -1 indicates first time
static int g_sIndTopItem = 0;
static int g_targetItem = 0;   // target item index for customization
static TablePtr g_pTable = NULL;
static int g_row = -1;
static tSyncTable *g_syncTable = NULL;

static void refreshTable(FormPtr frmP);
static int saveSyncTable(tSyncTable *t);
static void dropSyncTable(tSyncTable *t);
static tSyncTable *cacheSyncTable();
static void save();
static void jobNameDrawingProc(VoidPtr pTable, Int16 row, Int16 column, RectanglePtr pBounds);

static isy_BOOL SubsSetDetailsFormHandleEvent(EventPtr eventP);
static void SubsSetDetailsFormInit(FormPtr frmP, tSyncTable *t, int subsSetIndex);
static isy_BOOL doSubsSetDetailsForm(tSyncTable *t, int selectedRow);
static void saveSubsSetDetailsForm(FormPtr frm, tSyncTable *t, int subsSetIndex);

static void selectJobSyncMode(FormPtr frmP, VoidPtr pTable, int i);
static void updateFieldText(FieldPtr fld, isy_TCHAR *fieldText);

static int  doConfigReset(FormPtr frmP);

// Subscription Set Details form (new for v8.1 API)
static Boolean SubsSetDetailsFormHandleEvent(EventPtr eventP)
{
	Boolean handled = false;
	return handled;
	
} // SubsSetDetailsFormHandleEvent

static void SubsSetDetailsFormInit(FormPtr frmP, tSyncTable *t, int subsSetIndex)
{
	isy_TCHAR   subsSetId[ISCLEN_SubsSetID];
	isy_TCHAR   subsSetName[ISCLEN_SubsSetName];
	isy_WORD	w;
	isy_INT32	status;
	GET_APP_

	TSTR_CPY(subsSetName, stblJobName(t, subsSetIndex));
	TSTR_CPY(subsSetId, stblJobID(t, subsSetIndex));

	w = FrmGetObjectIndex(frmP, SubsSetDetailsNameLabel);
	FrmHideObject(frmP, w);
	FrmCopyLabel(frmP, SubsSetDetailsNameLabel, (char*) subsSetName);
	FrmShowObject(frmP, w);

	CtlSetValue(GetObjectPtr(frmP, SubsSetDetailsResetCheckbox), stblJobMode(t, subsSetIndex) == syncModeToItem(ISYNCMODE_Refresh));
	
	status = iscConfigGetSubsSetStatus(_pApp->hSyncConf, subsSetId);
	w = FrmGetObjectIndex(frmP, SubsSetDetailsStatusLabel);
	FrmHideObject(frmP, w);
	switch (status)
	{
		case ISCRTN_Succeeded :
			FrmCopyLabel(frmP, SubsSetDetailsStatusLabel, ISYNCSTR_Succeeded);			
			break;
		case ISCRTN_Canceled:
			FrmCopyLabel(frmP, SubsSetDetailsStatusLabel, ISYNCSTR_Cancelled);
			break;
		case ISCRTN_Failed:
			FrmCopyLabel(frmP, SubsSetDetailsStatusLabel, ISYNCSTR_Failed);
			break;
		case ISCRTN_Ready:
			FrmCopyLabel(frmP, SubsSetDetailsStatusLabel, ISYNCSTR_Ready);
			break;
		default:
			break;			
	}
	FrmShowObject(frmP, w);
	
	FrmCopyTitle(frmP, ISYNCSTR_Details);
	setCtrlLabel(frmP, SubsSetDetailsOkButton, ISYNCSTR_OK);
	setCtrlLabel(frmP, SubsSetDetailsCancelButton, ISYNCSTR_Cancel);
	setCtrlLabel(frmP, SubsSetDetailsResetCheckbox, ISYNCSTR_Reset);
	setCtrlLabel(frmP, SubsSetDetailsEnabledCheckbox, ISYNCSTR_Enable);
	setLabel(frmP, SubsSetDetailsTimeoutLabel, ISYNCSTR_SubsSet);
	setLabel(frmP, SubsSetDetailsUnnamed2205Label, ISYNCSTR_Status);
}

static void saveSubsSetDetailsForm(FormPtr frmP, tSyncTable *t, int subsSetIndex)
{
	char        subsSetId[ISCLEN_SubsSetID];
	GET_APP_

	TSTR_CPY(subsSetId, stblJobID(t, subsSetIndex));

/*
	if (CtlGetValue(GetObjectPtr(frmP, SubsSetDetailsEnabledCheckbox)) == 1) 
		syncConfigEnableSubsSet(_pApp->hSyncConf, subsSetId);
	else 
		syncConfigDisableSubsSet(_pApp->hSyncConf, subsSetId);
*/
		
	if (CtlGetValue(GetObjectPtr(frmP, SubsSetDetailsResetCheckbox)) == 1) 	
	{
		//iscConfigResetSubsSet(_pApp->hSyncConf, subsSetId);
		stblJobMode(t, subsSetIndex) = syncModeToItem(ISYNCMODE_Refresh);
	}
	else
		stblJobMode(t, subsSetIndex) = syncModeToItem(ISYNCMODE_Synchronize);

} // saveSubsSetDetailsForm


static Boolean doSubsSetDetailsForm(tSyncTable *t, int selectedRow)
{
	int subsSetIndex = selectedRow + g_indTopItem;

	FormPtr prevForm = FrmGetActiveForm();
	FormPtr frm = FrmInitForm(SubsSetDetailsForm);
	Word hitButton;
	
	FrmSetActiveForm(frm);
	FrmSetEventHandler(frm, SubsSetDetailsFormHandleEvent);
	FrmDrawForm(frm);
	SubsSetDetailsFormInit(frm, t, subsSetIndex);
		
	do hitButton = FrmDoDialog(frm);
	while (hitButton == SubsSetDetailsDefaultButton);
	
	if (hitButton == SubsSetDetailsOkButton)
		saveSubsSetDetailsForm(frm, t, subsSetIndex);

	if (prevForm) 
	{
		FrmSetActiveForm(prevForm);
		FrmDrawForm(prevForm);
	}
	FrmDeleteForm(frm);

	return hitButton == SubsSetDetailsOkButton;
}

static void dropSyncTable(tSyncTable *t)
{
	if (t == NULL) return;

	if (stblJobs(t)) {
		MEM_PTR_FREE(stblJobs(t));
		stblJobs(t) = NULL;
	}
	MEM_PTR_FREE(t);
} // dropSyncTable

static int saveSyncTable(tSyncTable *t)
{
	int i;
	int dirty;
	GET_APP_

	dirty = 0;
	for (i = 0; i < stblJobNum(t); i++) 
	{ 
		// skip the first job, which should be configjob
		unsigned long jMode = syncItemToMode(stblJobMode(t, i));
		if (stblJobModeDirty(t, i)) 
		{
			if (jMode == ISYNCMODE_Refresh)
				iscConfigResetSubsSet(_pApp->hSyncConf, stblJobID(t,i));							
			dirty = 1;	
		}
								
		if (stblJobEnabledDirty(t, i)) 
		{
			if (stblJobEnabled(t, i))
				iscConfigEnableSubsSet(_pApp->hSyncConf, stblJobID(t,i));	
			else						
				iscConfigDisableSubsSet(_pApp->hSyncConf, stblJobID(t,i));	
			dirty = 1;	
		}
	} // for each job, including configjob
	
	if (stblJobs(t)) 
	{
		MEM_PTR_FREE(stblJobs(t));
		stblJobs(t) = NULL;
	}
	MEM_PTR_FREE(t);

	return dirty;
} // saveSyncTable

static tSyncTable *cacheSyncTable()
{
	tSyncTable *new = (tSyncTable *) MEM_PTR_ALLOC(sizeof(tSyncTable));
	unsigned long uMode;
	int         i;
	isy_INT32	reset;
	
	isy_TCHAR	id[ISCLEN_SubsSetID];
	isy_TCHAR	name[ISCLEN_SubsSetName];
	HISCCSR	hCursor;	
	GET_APP_
	
	stblJobNum(new) = 0;
	
	iscConfigOpenCursor(_pApp->hSyncConf, &hCursor);	
	while( iscConfigGetNextSubsSet(_pApp->hSyncConf, hCursor, id, name) == ISCRTN_Succeeded )
	{
		stblJobNum(new)++;
	}
	iscConfigCloseCursor(_pApp->hSyncConf, hCursor);
	stblJobs(new) = (tJobSyncTable *) MEM_PTR_ALLOC(sizeof(tJobSyncTable) * stblJobNum(new));
	
	i = 0;
	iscConfigOpenCursor(_pApp->hSyncConf, &hCursor);	
	while( iscConfigGetNextSubsSet(_pApp->hSyncConf, hCursor, id, name) == ISCRTN_Succeeded )
	{
		if (STR_CMP(id, "configjob") ==0) {
			//i--; // skip configjob
			stblJobNum(new)--;
			continue;
		}
		reset = iscConfigSubsSetIsReset(_pApp->hSyncConf, id);
		uMode = reset ? ISYNCMODE_Refresh : ISYNCMODE_Synchronize;
				
		TSTR_CPY(stblJobID(new, i), id);
		TSTR_CPY(stblJobName(new, i), name);
		stblSubsNum(new, i) = -1; // indicates never visited 
		stblJobEnabled(new, i) = stblJobOldEnabled(new, i) = iscConfigSubsSetIsEnabled(_pApp->hSyncConf, id);
		stblJobMode(new, i) = stblJobOldMode(new, i) = syncModeToItem(uMode);
		i++;
	}
	
	iscConfigCloseCursor(_pApp->hSyncConf, hCursor);
	
	return new;
} // cacheSyncTable


static void formInit(FormPtr frmP)
{
	isy_TCHAR  URL[ISYNCLEN_URL];
	GET_APP_
	
	g_cItems = -1;
	
	refreshTable(frmP);
	
	appGetPref(ISYNCPREF_URL, URL, ISYNCLEN_URL);
          
	FrmCopyLabel(frmP, OptionsCurrentServerLabel, URL);

	// Locally populated subscriptions
	FrmHideObject(frmP, FrmGetObjectIndex(frmP, OptionsUpdateListCheckbox));
	FrmHideObject(frmP, FrmGetObjectIndex(frmP, OptionsNewButton));
	FrmHideObject(frmP, FrmGetObjectIndex(frmP, OptionsEditButton));
	FrmHideObject(frmP, FrmGetObjectIndex(frmP, OptionsDeleteButton));

	FrmCopyTitle(frmP, ISYNCSTR_SubsSet_Settings);
	setCtrlLabel(frmP, OptionsDoneButton, ISYNCSTR_OK);
	setCtrlLabel(frmP, OptionsCancelButton, ISYNCSTR_Cancel);
	setCtrlLabel(frmP, OptionsDetailsButton, ISYNCSTR_Details);
	setCtrlLabel(frmP, OptionsResetButton, ISYNCSTR_Purge);
	setLabel(frmP, OptionsUnnamed1202Label, "");
	setLabel(frmP, OptionsUnnamed1214Label, ISYNCSTR_Server);
}

static Boolean formDoCommand(Word command)
{
	Boolean handled = false;
	
	switch (command) {
	default:
	break;
	}
	
	return handled;
}

static void selectJobSyncMode(FormPtr frmP, VoidPtr pTable, int i)
{
	int row = i + g_indTopItem;
	int iMode = TblGetItemInt(pTable, i, 0);
	int oldMode = stblJobOldMode(g_syncTable, row);
	
	
	if(TblGetItemInt(pTable, i, 0) == 1)
		stblJobEnabled(g_syncTable, row) = true;
	else
		stblJobEnabled(g_syncTable, row) = false;
} // selectJobSyncMode

		                
static void refreshTable(FormPtr frmP)
{
	ScrollBarPtr pScroll;
	TablePtr pTable;
	ListPtr  pList3;
	int      rows;
	int      i;
	int		 n;
	int      iMode;
	int      recordIndex;        // starts at 0
	
	n = 0;
	pScroll = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, OptionsScrollScrollBar));
	pTable  = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, OptionsDetailsTable));
	rows    = TblGetNumberOfRows(pTable);
	g_cVisibles = rows;
	
	if (g_cItems == -1) { // the first time
		if (g_syncTable == NULL) g_syncTable = cacheSyncTable();
		g_indTopItem = 0;
	}
	recordIndex = g_indTopItem;
	g_ridTopItem = recordIndex;
	
	for (i = 0; i < rows; i++) {
		TblSetItemStyle(pTable, i, 0, checkboxTableItem);
		TblSetItemStyle(pTable, i, 1, customTableItem);
		
		if (i >= stblJobNum(g_syncTable)) {
			TblSetItemInt(pTable, i, 0, 0);
			TblSetRowData(pTable, i, (ULong) -1);
			TblSetRowUsable(pTable, i, false);
			continue;
		}
		
		if(stblJobEnabled(g_syncTable, recordIndex))
			TblSetItemInt(pTable, i, 0, 1);
		else
		        TblSetItemInt(pTable, i, 0, 0);
			
		
		TblSetRowData(pTable, i, (ULong) recordIndex);
		TblSetRowUsable(pTable, i, true);
		
		++recordIndex;
	}
	TblSetCustomDrawProcedure(pTable, 1, (TableDrawItemFuncPtr) jobNameDrawingProc);
	TblSetColumnUsable(pTable, 0, true);
	TblSetColumnUsable(pTable, 1, true);
	
	if (g_cItems == -1)     // first time, need to count # of items
	{
		g_cItems = stblJobNum(g_syncTable);
		n = g_cItems - rows;
		if (n < 0) n = 0;
		SclSetScrollBar(pScroll, g_indTopItem, 0, n, rows);
	}
	
	// TblUnhighlightSelection(pTable);        // still necessary - this rectangle exceeds somewhat the table
	TblMarkTableInvalid(pTable);
	TblRedrawTable(pTable);
}

static void jobNameDrawingProc(VoidPtr pTable, Int16 row, Int16 column, RectanglePtr pBounds)
{
	FontID   	fnt;
	char     	name[ISCLEN_SubsSetName];
	int      	recordIndex;        // starts at 0
	
	WinEraseRectangle(pBounds, 0);
	recordIndex = (int) TblGetRowData(pTable, row);
	if (recordIndex == -1) return;
	
	TSTR_CPY(name, stblJobName(g_syncTable, recordIndex));
	
	fnt = FntSetFont(boldFont);
	WinDrawChars(name, StrLen(name), pBounds->topLeft.x + FntCharsWidth(" ", 1), pBounds->topLeft.y);
	FntSetFont(fnt);
}

static void save()
{
	FormPtr  frmP = FrmGetActiveForm();
	GET_APP_
	
	saveSyncTable(g_syncTable);
}

static void updateFieldText(FieldPtr fld, isy_TCHAR *fieldText)
{
	Handle h = MemHandleNew((TSTR_LEN(fieldText) + 1) * sizeof(isy_TCHAR));
	Handle hOld = FldGetTextHandle(fld);
	isy_TCHAR *text = (isy_TCHAR *) MemHandleLock(h);

	TSTR_CPY(text, fieldText);
	MemHandleUnlock(h);
	FldSetTextHandle(fld, h);
	FldDrawField(fld);
	if (hOld) MemHandleFree(hOld);
} // updateFieldText


Boolean OptionsFormHandleEvent(EventPtr eventP)
{
	Boolean  handled = false;
	FormPtr  frmP;
	TablePtr pTable;
	int      i;
	int rc = 0;
	
	switch (eventP->eType) 
	{
	case menuEvent:
		return formDoCommand(eventP->data.menu.itemID);
	case frmOpenEvent:
		rc = syncStartup(NULL);    
		frmP = FrmGetActiveForm();
		if (rc == ISCRTN_Succeeded) 
		{
		formInit(frmP);
		FrmDrawForm(frmP);
		}
		else if (rc != ISCRTN_Succeeded) 
		{
		isy_TCHAR statusMsg[128];
			statusMsg[0] = '\0';
			TSTR_CPY(statusMsg, ISYNCSTR_StartupError);
			TSTR_ITOA(statusMsg + TSTR_LEN(statusMsg), rc);
			appMessageBox(statusMsg, APPMB_OK);	
			FrmGotoForm(MainForm);	        
		}
		
		handled = true;
		break;
	case ctlSelectEvent:
		frmP = FrmGetActiveForm();
		switch (eventP->data.ctlSelect.controlID) 
		{
		case OptionsDoneButton:
		        save();
					g_syncTable = NULL;
					selectedRow = -1;
		        FrmGotoForm(MainForm);
		        handled = true;
		        break;
		case OptionsCancelButton: // no save
		        dropSyncTable(g_syncTable);
					g_syncTable = NULL;
					selectedRow = -1;
		        FrmGotoForm(MainForm);
		        handled = true;
		        break;
			case OptionsDetailsButton:
					handled = true;
		    	if (selectedRow != -1) 
						doSubsSetDetailsForm(g_syncTable, selectedRow);
					break;           
			case OptionsResetButton:
		            doConfigReset(frmP);
		            handled = true;
		            break;             
		}
		break;
	case sclRepeatEvent:
		if (g_indTopItem != eventP->data.sclRepeat.newValue)
		{
		        g_indTopItem = eventP->data.sclRepeat.newValue;
		        refreshTable(FrmGetActiveForm());
		        handled = false;        // scrollbar will handle this event as well
		}
		break;
	case tblEnterEvent:
		//frmP = FrmGetActiveForm();
		g_pTable = pTable = eventP->data.tblEnter.pTable;
		g_row = i = eventP->data.tblSelect.row;
		handled = false;
		break;
	case winExitEvent:
		//frmP = FrmGetActiveForm();
			if (g_pTable) 
			{
				g_pTable = NULL;
				g_row = -1;
			} // if (g_pTable)
		handled = false;
		break;
	case tblSelectEvent:
		//frmP = FrmGetActiveForm();
		pTable = eventP->data.tblSelect.pTable;
		i = eventP->data.tblSelect.row;
		selectedRow = eventP->data.tblSelect.row;
		selectJobSyncMode(NULL, pTable, i);
		handled = true;
		break;
	case frmCloseEvent:
		dropSyncTable(g_syncTable);
		handled = false;
		syncShutdown();   	    
		break;
	default:
		handled = false;
		break;	
	}
	
	return handled;
}

static int doConfigReset(FormPtr frmP)
{
	int okCancel = appMessageBox(ISYNCSTR_ResetConfig, APPMB_OKCANCEL);
	int rc;
	GET_APP_
	
	rc = syncStartup(NULL);
	if (rc == ISCRTN_Succeeded) 
	{	
		if (okCancel == 1) 
		{
			selectedRow = -1;
	        dropSyncTable(g_syncTable);
			g_syncTable = NULL;		
			iscConfigPurge(_pApp->hSyncConf);
	        FrmGotoForm(MainForm);
		}
	}
	else
	{
		isy_TCHAR statusMsg[128];
		statusMsg[0] = '\0';
		TSTR_CPY(statusMsg, ISYNCSTR_StartupError);
		TSTR_ITOA(statusMsg + TSTR_LEN(statusMsg), rc);
		appMessageBox(statusMsg, APPMB_OK);	
	}
	
	syncShutdown();
	return okCancel;
} // doConfigReset