/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */
/************************************************************************
 *
 * File Name:  Starter.c
 * Description: Palm-specific application main program
 *
 ************************************************************************
 */

#ifdef UNDER_PO_31
#	include <Pilot.h>
#else
#	include <PalmOS.h>
#	include <PalmCompatibility.h>
#endif
#include <SysEvtMgr.h>
#include "StarterRsc.h"

#include "app.h"
#include "MainForm.h"
#include "SyncForm.h"
#include "OptionsForm.h"
#include "LogForm.h"

#define version20 0x02000000

static Err RomVersionCompatible(DWord requiredVersion, Word launchFlags)
{
	DWord romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
	{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
		{
			FrmAlert (RomIncompatibleAlert);
		
			// Pilot 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion < version20)
			{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
			}
		}
		
		return (sysErrRomIncompatible);
	}

	return (0);
}

static Boolean AppHandleEvent( EventPtr eventP)
{
	Word formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
	{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId) {
		case MainForm:
			FrmSetEventHandler(frmP, MainFormHandleEvent);
			break;
		case SyncForm:
			FrmSetEventHandler(frmP, SyncFormHandleEvent);
			break;
		case LogForm:
			FrmSetEventHandler(frmP, LogFormHandleEvent);
			break;
		case OptionsForm:
			FrmSetEventHandler(frmP, OptionsFormHandleEvent);
			break;
		default:
			break;

		}
		
		return true;
	}
	
	return false;
}

static void AppEventLoop(void)
{
	Word error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);
		
		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}

static Err AppStart(void)
{
	Err error;
	GET_APP_
	
	appInitialize();
	
	if(((error = DmCreateDatabase(0, "server-isyn", appFileCreator, 'sevr', false)) != dmErrAlreadyExists) && (error != 0))
						ErrDisplay("Error creating database!");
	
    return 0;
}

static void AppStop(void)
{
    FrmSaveAllForms();
    FrmCloseAllForms();
	appTerminate();
}

static DWord StarterPilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	Err error;
	
	GET_APP_

	error = RomVersionCompatible (version20, launchFlags);
	if (error) return (error);

	switch (cmd) {
	  // intended for the HotSync button on the craddle
	  case sysAppLaunchCmdSyncRequestRemote:
	  // intended for the HotSync button on the modem
	  case sysAppLaunchCmdSyncRequestLocal:
		// invoked by either HotSync button
		_pApp->hotSync = true;
		// fall through
	  case sysAppLaunchCmdNormalLaunch:
	  	if(cmdPBP != NULL)
	  	{
	  		if(StrCompare((char *)cmdPBP, "SyncNow") == 0)
	  			_pApp->hotSync = true;
	  	}
		error = AppStart();
		if (error) 
			return error;
				
		FrmGotoForm(MainForm);
		AppEventLoop();
		AppStop();
		break;

	  default:
		break;

	} // switch (cmd)
	
	return 0;
}

DWord PilotMain( Word cmd, Ptr cmdPBP, Word launchFlags)
{
    return StarterPilotMain(cmd, cmdPBP, launchFlags);
}
