/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  Dialogs.cpp
 *
 * Description:
 *    Dialog box classes implementation
 *
 */

#include "Dialogs.h"

// Eikon.
#include <Eikclb.h>
#include <eikbtpan.h>	// Button panel.
#include <eikcmbut.h>	// Command button.
#include <frmtlay.h>

static isy_UINT32 parseMsgSizeFromBaud(isy_TCHAR *selNetType, int sel);

//======================================================================
// CLoginDialog
CLoginDialog::CLoginDialog(ISCLISTENARG *args)
{
	this->args = args;
}

CLoginDialog::~CLoginDialog()
{
}

void CLoginDialog::PreLayoutDynInitL()
{
	TBuf16<ISYNCLEN_Target> targetBuf;
	CEikEdwin* edwin = STATIC_CAST(CEikEdwin*, Control(ELoginTargetEditor));
	targetBuf = TPtrC16(args->argv[0], TSTR_LEN(args->argv[0]));
	edwin->SetTextL(&targetBuf);
	edwin->SetReadOnly(ETrue);
}

TBool CLoginDialog::OkToExitL(TInt aKeycode)
{
	EPOCGLOBALS* globals = epocGetGlobals();
	EPUIDATA* dataPtr = (EPUIDATA*) globals->hData;

	if (aKeycode == EEikBidOk) 
	{
		CEikEdwin* edwin;
		isy_TCHAR buf[ISYNCLEN_User];
	
		edwin=STATIC_CAST(CEikEdwin*, Control(ELoginUserEditor));
		HBufC* buf1=edwin->GetTextInHBufL();
		DesCToCharPtr(buf1, buf);
		TSTR_CPY(args->argv[1], buf);
		delete buf1;

		edwin=STATIC_CAST(CEikEdwin*, Control(ELoginPasswordEditor));
		HBufC* buf2=edwin->GetTextInHBufL();
		DesCToCharPtr(buf2, buf);
		TSTR_CPY(args->argv[2], buf);
		delete buf2;

		dataPtr->msgBoxButtonClicked = MessageOKButton;
		return ETrue;
	}
	else
	{
		TSTR_CPY(args->argv[1], isy_T(""));
		TSTR_CPY(args->argv[2], isy_T(""));
	}

	dataPtr->msgBoxButtonClicked = MessageCancelButton;	
	return ETrue;
}

//======================================================================
// CServerSettingsDialog
CServerSettingsDialog::CServerSettingsDialog()
{
}

CServerSettingsDialog::~CServerSettingsDialog()
{
}


// This function is called just before the dialog is displayed. It allows the
// dialog to be initialised with application data - in this case, the current
// co-ordinates stored in the model.
void CServerSettingsDialog::PreLayoutDynInitL()
{
	// Initialise the numeric controls in the dialog
	// with the data from the model 

	CEikEdwin* edwin;
	CEikSecretEditor* secret_edwin;
	TBuf16<ISYNCLEN_MiscInfo> buffer;
	isy_INT status;
	
	ISYNCPREFS prefs;
	appGetPref(ISYNCPREF_URL,      prefs.URL, sizeof(prefs.URL)/sizeof(isy_TCHAR));			// Protocol://IP or hostname:port
	appGetPref(ISYNCPREF_User,     prefs.user, sizeof(prefs.user)/sizeof(isy_TCHAR));			// user ID
	appGetPref(ISYNCPREF_Password, prefs.password, sizeof(prefs.password)/sizeof(isy_TCHAR));	// password
	status = appGetPref(ISYNCPREF_UnsavedPassword, prefs.password, ISYNCLEN_Password);

	TBuf16<ISYNCLEN_MiscInfo> buf1;
	edwin=STATIC_CAST(CEikEdwin*, Control(ESettingsServerIPEditor));
	buf1 = TPtrC16(&prefs.URL[0], TSTR_LEN(prefs.URL));
	edwin->SetTextL(&buf1);

	edwin=STATIC_CAST(CEikEdwin*, Control(ESettingsNameEditor));
	buf1 = TPtrC16(&prefs.user[0], TSTR_LEN(prefs.user));
	edwin->SetTextL(&buf1);

	secret_edwin = STATIC_CAST(CEikSecretEditor*, Control(ESettingsPasswordEditor));
	buf1 = TPtrC16(&prefs.password[0], TSTR_LEN(prefs.password));
	secret_edwin->SetText(buf1);

	if (status == ISCRTN_True)
	{ 
		CharPtrToFieldDes(prefs.password, buffer);
		iCheckboxState = CEikButtonBase::ESet; 
	}
	else
	{ 
		iCheckboxState = CEikButtonBase::EClear; 
	}

	CEikCheckBox* checkBox=static_cast<CEikCheckBox*>(Control(ESavePasswordCheckBox));
	checkBox->SetState(iCheckboxState);
	checkBox->DrawNow();
}

// This function is called whenever a button is pressed in the dialog's button
// panel (or a button is activated using its hotkey). It is not called for
// the Cancel button.
TBool CServerSettingsDialog::OkToExitL(TInt aKeycode)
{
	if (aKeycode == EEikBidOk) 
	{
		isy_TCHAR charBuf[ISYNCLEN_MiscInfo];
		TBuf<ISYNCLEN_MiscInfo> buffer;
		HBufC* buf;
	
		CEikEdwin* edwin;
		CEikSecretEditor* secret_edwin;

		ISYNCPREFS prefs;

		edwin=STATIC_CAST(CEikEdwin*, Control(ESettingsServerIPEditor));
		HBufC* buf2=edwin->GetTextInHBufL();
		DesCToCharPtr(buf2, charBuf);
		TSTR_CPY(prefs.URL, charBuf);
		delete buf2;

		edwin=STATIC_CAST(CEikEdwin*, Control(ESettingsNameEditor));
		HBufC* buf4=edwin->GetTextInHBufL();
		DesCToCharPtr(buf4, charBuf);
		TSTR_CPY(prefs.user, charBuf);
		delete buf4;
		
		appSetPref(ISYNCPREF_URL,      prefs.URL);		// Protocol://IP or hostname:port
		appSetPref(ISYNCPREF_User,     prefs.user);		// user ID
	
		secret_edwin = STATIC_CAST(CEikSecretEditor*, Control(ESettingsPasswordEditor));
		secret_edwin->GetText(buffer);
		buf = buffer.AllocL();
		DesCToCharPtr(buf, charBuf);
		TSTR_CPY(prefs.password, charBuf);

		CEikCheckBox* checkBox=static_cast<CEikCheckBox*>(Control(ESavePasswordCheckBox));
		iCheckboxState = checkBox->State();
		if(iCheckboxState == CEikButtonBase::ESet)
		{
			appSetPref(ISYNCPREF_Password, prefs.password);
		}
		else
		{
			appSetPref(ISYNCPREF_UnsavedPassword, prefs.password);
		}

		delete buf;


		return ETrue;
	}
	else if (aKeycode == EClientSettingsButton) 
	{
		CClientSettingsDialog* dialog=new (ELeave) CClientSettingsDialog();
		dialog->ExecuteLD(R_CLIENT_SETTINGS_DIALOG);
		return EFalse;
	}
	else if (aKeycode == ENetworkSettingsButton)
	{
		CNetworkSettingsDialog* dialog=new (ELeave) CNetworkSettingsDialog();
		dialog->ExecuteLD(R_NETWORK_SETTINGS_DIALOG);
		return EFalse;
	}

	return EFalse;
}

//======================================================================
// CClientSettingsDialog
CClientSettingsDialog::CClientSettingsDialog()
{
}

CClientSettingsDialog::~CClientSettingsDialog()
{
}

void CClientSettingsDialog::PreLayoutDynInitL()
{ 
	isy_TCHAR traceOn[2];
	isy_TCHAR targetDir[ISYNCLEN_Target];
	TBuf16<ISYNCLEN_MiscInfo> buf1;
	CEikEdwin* edwin;

	appGetPref(ISYNCPREF_Trace, traceOn, sizeof(traceOn)/sizeof(isy_TCHAR));

	// display trace checkbox
	if (*traceOn == 'Y')
	{ 
		iCheckboxState = CEikButtonBase::ESet; 
		//SetCheckBoxState(EDetailedLogCheckBox, iCheckboxState);
	}
	else
	{ 
		iCheckboxState = CEikButtonBase::EClear; 
		//SetCheckBoxState(EDetailedLogCheckBox, iCheckboxState);
	}

	CEikCheckBox* checkBox=static_cast<CEikCheckBox*>(Control(EDetailedLogCheckBox));
	checkBox->SetState(iCheckboxState);
	checkBox->DrawNow();
	// end trace checkbox

	//display target directory
	appGetPref(ISYNCPREF_TargetDir, targetDir, sizeof(targetDir)/sizeof(isy_TCHAR));
	if(TSTR_LEN(targetDir) == 0)
		TSTR_CPY(targetDir, DATADIR);

	edwin=STATIC_CAST(CEikEdwin*, Control(ETargetDirEditor));
	buf1 = TPtrC16(targetDir, TSTR_LEN(targetDir));
	edwin->SetTextL(&buf1);
	//end target directory
}

TBool CClientSettingsDialog::OkToExitL(TInt )
{
	isy_TCHAR traceOn[2];
	CEikEdwin* edwin;
	isy_TCHAR charBuf[ISYNCLEN_Target];
	isy_TCHAR targetDir[ISYNCLEN_Target];

	//iCheckboxState = CheckBoxState(EDetailedLogCheckBox);
	CEikCheckBox* checkBox=static_cast<CEikCheckBox*>(Control(EDetailedLogCheckBox));
	iCheckboxState = checkBox->State();
	
	//Save trace checkbox
	if(iCheckboxState == CEikButtonBase::ESet)
	{
		TSTR_CPY(traceOn, ISCCONST_TraceON);
		appSetPref(ISYNCPREF_Trace, traceOn);
	}
	else
	{
		TSTR_CPY(traceOn, ISCCONST_TraceOFF);
		appSetPref(ISYNCPREF_Trace, traceOn);
	}
	//end save trace checkbox

	//save target directory
	edwin=STATIC_CAST(CEikEdwin*, Control(ETargetDirEditor));
	HBufC* buf1=edwin->GetTextInHBufL();
	DesCToCharPtr(buf1, charBuf);
	TSTR_CPY(targetDir, charBuf);
	appSetPref(ISYNCPREF_TargetDir, targetDir);
	delete buf1;
	//end save target directory

	return ETrue;
}

//======================================================================
// CNetworkSettingsDialog
CNetworkSettingsDialog::CNetworkSettingsDialog()
{
	iCheckboxState = CEikButtonBase::EClear;
}

CNetworkSettingsDialog::~CNetworkSettingsDialog()
{
}

void CNetworkSettingsDialog::PreLayoutDynInitL()
{ 
	EPOCGLOBALS* globals = epocGetGlobals();
	APPDATA* appPtr = (APPDATA*) globals->hApp;

	isy_TCHAR buf[ISYNCLEN_MiscInfo];
	int value = 0, sel = 0, i = 0;
	int numTimeoutValues = 19;
	int numNetworkValues = 6;

	TBuf16<ISYNCLEN_MiscInfo> buf1;
	CEikEdwin* edwin;

	// Initialize the proxy
	if (appPrefOptionProxy(appPtr))
	{
		iCheckboxState = CEikButtonBase::ESet; 
	}
	else
	{
		iCheckboxState = CEikButtonBase::EClear; 
	}
	CEikCheckBox* checkBox=static_cast<CEikCheckBox*>(Control(EProxyCheckBox));
	checkBox->SetState(iCheckboxState);
	checkBox->DrawNow();

	isy_TCHAR proxyDest[ISYNCLEN_Server+ISYNCLEN_Port] = {0};
	appGetPref(ISYNCPREF_Proxy, proxyDest, sizeof(proxyDest)/sizeof(isy_TCHAR));
	if (proxyDest)
	{
		isy_TCHAR *proxyPort = (isy_TCHAR*) TSTR_CHR(proxyDest, (isy_TCHAR)':');
		if (proxyPort)
		{
			edwin=STATIC_CAST(CEikEdwin*, Control(EProxyPortEditor));
			buf1 = TPtrC16(proxyPort+1, TSTR_LEN(proxyPort+1));
			edwin->SetTextL(&buf1);

			proxyDest[proxyPort-proxyDest] = (isy_TCHAR)'\0';

			edwin=STATIC_CAST(CEikEdwin*, Control(EProxyHostEditor));
			buf1 = TPtrC16(proxyDest, TSTR_LEN(proxyDest));
			edwin->SetTextL(&buf1);
		}
	}
	// end proxy

	// display the timeout combo box
	MEM_SET(buf, 0, sizeof(buf));
	appGetPref(ISYNCPREF_TimeoutRecv, buf, sizeof(buf)/sizeof(isy_TCHAR));

	value = TSTR_ATOI(buf);
	if (value > 0) value = value / 60;
	if (value < 0) sel = numTimeoutValues - 1;  //(never)
	else if (value == 0) sel = 0; //(minimum)
	else 
	{
		if (value <= 10) sel = value;
		else if (value <= 30) sel = (value-10)/5+10;
		else sel = value/10-4+15;
	}

	//SetChoiceListCurrentItem(ETimeoutActionPopup, sel);
	CEikChoiceList* choiceList = static_cast<CEikChoiceList*>(Control(ETimeoutActionPopup));
	choiceList->SetCurrentItem(sel);
	choiceList->DrawNow();
	// end timeout combo box

	// Initialize the network combo box
	CEikChoiceList* choiceListNetwork = static_cast<CEikChoiceList*>(Control(ENetworkActionPopup));

	MEM_SET(buf, 0, sizeof(buf));
	appGetPref(ISYNCPREF_Message_Size, buf, sizeof(buf)/sizeof(isy_TCHAR));
	value = TSTR_ATOI(buf);

	if (value <= 0)
		sel = 0;		// default is unset, or, let engine calculate
	else
	{
		int netBaudInt = 0;
		CDesCArray* choices = choiceListNetwork->DesCArray();
		for (i = 1; i < numNetworkValues-1; i++)
		{
			TPtrC16 tPtr = choices->operator[](i);
			TSTR_N_CPY(buf, tPtr.Ptr(), tPtr.Length());
			buf[tPtr.Length()] = (isy_TCHAR) '\0';
			netBaudInt = parseMsgSizeFromBaud(buf, i);
			if (value <= netBaudInt)
				break;
		}
		sel = i;
	}

	choiceListNetwork->SetCurrentItem(sel);
	choiceListNetwork->DrawNow();
	// end network combo box
}

TBool CNetworkSettingsDialog::OkToExitL(TInt )
{
	EPOCGLOBALS* globals = epocGetGlobals();
	APPDATA* appPtr = (APPDATA*) globals->hApp;
	CEikEdwin* edwin;

	int numTimeoutValues = 19;
	isy_TCHAR buf[ISYNCLEN_MiscInfo];
	int value = 0, sel = 0;

	//Save proxy
	CEikCheckBox* checkBox=static_cast<CEikCheckBox*>(Control(EProxyCheckBox));
	iCheckboxState = checkBox->State();

	if(iCheckboxState == CEikButtonBase::ESet)
	{
		isy_TCHAR proxyDest[ISYNCLEN_Server+ISYNCLEN_Port] = {0};
		appSetPrefOptionProxy(appPtr);

		edwin=STATIC_CAST(CEikEdwin*, Control(EProxyHostEditor));
		HBufC* buf1=edwin->GetTextInHBufL();
		DesCToCharPtr(buf1, buf);
		delete buf1;

		TSTR_CPY(proxyDest, buf);
		TSTR_CAT(proxyDest, isy_T(":"));

		edwin=STATIC_CAST(CEikEdwin*, Control(EProxyPortEditor));
		HBufC* buf2=edwin->GetTextInHBufL();
		DesCToCharPtr(buf2, buf);
		delete buf2;

		TSTR_CAT(proxyDest, buf);
		appSetPref(ISYNCPREF_Proxy, proxyDest);
	} 
	else 
	{
		appResetPrefOptionProxy(appPtr);
	}
	//end save proxy

	//Save timeout combo box
	//sel = ChoiceListCurrentItem(ETimeoutActionPopup);
	CEikChoiceList* choiceList = static_cast<CEikChoiceList*>(Control(ETimeoutActionPopup));
	sel = choiceList->CurrentItem();
	
	if (sel == 0) TSTR_CPY(buf, isy_T("0"));
	else if (sel == (numTimeoutValues-1)) TSTR_CPY(buf, isy_T("-1"));
	else 
	{
		if (sel <= 10) TSTR_ITOA(buf, sel);
		else if (sel < 15) TSTR_ITOA(buf, (sel-10)*5+10);
		else TSTR_ITOA(buf, (sel+4-15)*10);
		value = TSTR_ATOI(buf);
		TSTR_ITOA(buf, value * 60);
	}
	appSetPref(ISYNCPREF_TimeoutRecv, buf); // Recv timeout
	appSetPref(ISYNCPREF_TimeoutSend, buf); // Send timeout
	//end save timeout combo box

	//Save network combo box
	CEikChoiceList* choiceListNetwork = static_cast<CEikChoiceList*>(Control(ENetworkActionPopup));
	CDesCArray* choices = choiceListNetwork->DesCArray();
	sel = choiceListNetwork->CurrentItem();

	TPtrC16 tPtr = choices->operator[](sel);
	TSTR_N_CPY(buf, tPtr.Ptr(), tPtr.Length());
	buf[tPtr.Length()] = (isy_TCHAR) '\0';
	value = parseMsgSizeFromBaud(buf, sel);
	TSTR_ITOA(buf, value);
	appSetPref(ISYNCPREF_Message_Size, buf);
	//end save network combo box

	return ETrue;
}

void CNetworkSettingsDialog::HandleControlStateChangeL(TInt aControlId)
{
	switch(aControlId)
	{
	case ETimeoutActionPopup:
		break;
	default:
		break;
	}
}

static isy_UINT32 parseMsgSizeFromBaud(isy_TCHAR *selNetType, int sel)
{
	isy_TCHAR *space = (isy_TCHAR*) TSTR_CHR(selNetType, (isy_TCHAR)' ');
	isy_TCHAR baudIntStr[12];
	long multiplier = 1024;
	isy_TCHAR *m;

	EPOCGLOBALS* globals = epocGetGlobals();
	EPUIDATA* dataPtr = (EPUIDATA*) globals->hData;
	
	// 1st entry is "none" or disabled.  Protect against malformed formats
	if (sel == 0 || space == NULL || (space - selNetType)-1 > (sizeof(baudIntStr)/sizeof(isy_TCHAR)))
		return 0;
		
	{
		int len = space - selNetType, baudInt;
		
		// extract the baud int, e.g. 4, 56, 2 (Meg)
		TSTR_N_CPY(baudIntStr, selNetType, len);
		baudIntStr[len] = (isy_TCHAR)'\0';
		baudInt = TSTR_ATOI(baudIntStr);
		if (baudInt <= 0 || baudInt > 10 * 1000 * 1024)
			dataPtr->messageBox(isy_T("Internal network type Baud Rate"), dataPtr->engine, APPMB_OK, NULL);
	    
		// Now extract the multiplier, e.g. K, M
		for (m = space; *m != '\0'; m++)
		{
			if (*m == 'K' || *m == 'M')
				break;
		}
		if (*m == 'M')
		    	multiplier *= 1000;
	    	
		return (baudInt * multiplier) / 4;
	}
}

//======================================================================
// CPasswordDialog
CPasswordDialog::CPasswordDialog()
{
	iCheckboxState = CEikButtonBase::ESet;
	iCheckBox = new(ELeave) CEikCheckBox;
	iCheckBox->SetContainerWindowL(*this);
	//iCheckBox->ConstructL();
}

CPasswordDialog::~CPasswordDialog()
{
	delete iCheckBox;
}

// This function is called just before the dialog is displayed. It allows the
// dialog to be initialised with application data - in this case, the current
// co-ordinates stored in the model.
void CPasswordDialog::PreLayoutDynInitL()
{ 
	TBuf16<ISYNCLEN_MiscInfo> buffer;
	
	ISYNCPREFS prefs;
	isy_INT status;
	status = appGetPref(ISYNCPREF_UnsavedPassword, prefs.password, ISYNCLEN_Password);

	if (status == ISCRTN_True)
	{ 
		CharPtrToFieldDes(prefs.password, buffer);
		iCheckboxState = CEikButtonBase::ESet; 
	}
	else // display UNCHECKED
	{ 
		iCheckboxState = CEikButtonBase::EClear; 
	}

	CEikCheckBox* checkBox=static_cast<CEikCheckBox*>(Control(ESavePasswordCheckBox));
	checkBox->SetState(iCheckboxState);
	checkBox->DrawNow();
}

// This function is called whenever a button is pressed in the dialog's button
// panel (or a button is activated using its hotkey).
TBool CPasswordDialog::OkToExitL(TInt )
{
	TBuf<ISYNCLEN_MiscInfo> buffer;
	HBufC* buf;
	isy_TCHAR charBuf[ISYNCLEN_MiscInfo];
	CEikSecretEditor* edwin;
	
	ISYNCPREFS prefs;
	
	edwin = STATIC_CAST(CEikSecretEditor*, Control(ESettingsPasswordEditor));
	edwin->GetText(buffer);
	buf = buffer.AllocL();
	DesCToCharPtr(buf, charBuf);
	TSTR_CPY(prefs.password, charBuf);

	CEikCheckBox* checkBox=static_cast<CEikCheckBox*>(Control(ESavePasswordCheckBox));
	iCheckboxState = checkBox->State();
	if(iCheckboxState == CEikButtonBase::ESet)
	{
		appSetPref(ISYNCPREF_Password, prefs.password);
	}
	else
	{
		appSetPref(ISYNCPREF_UnsavedPassword, prefs.password);
	}

	delete buf;
	return ETrue;
}

//======================================================================
// CDetailsDialog
CDetailsDialog::CDetailsDialog(TInt index)
{
	iCheckboxState = CEikButtonBase::EClear;
	listIndex = index;
}

CDetailsDialog::~CDetailsDialog()
{
}

void CDetailsDialog::PreLayoutDynInitL()
{ 
	CEikEdwin* edwin;
	TBuf16<ISYNCLEN_MiscInfo> buf1, buf2;
	
	HISCCSR	hCursor;
	isy_INT32 rc = ISCRTN_Succeeded, status;
	isy_TCHAR statusBuf[16];

	EPOCGLOBALS* globals = epocGetGlobals();
	APPDATA* appPtr = (APPDATA*) globals->hApp;

	iscConfigOpenCursor(appPtr->hSyncConf, &hCursor);
	for (int i=0; i <= listIndex; i++)
	{
		rc = iscConfigGetNextSubsSet(appPtr->hSyncConf, hCursor, id, name);
		if (rc != ISCRTN_Succeeded) 
			break;
	}
	iscConfigCloseCursor(appPtr->hSyncConf, hCursor);

	edwin=STATIC_CAST(CEikEdwin*, Control(EDetailsSubsSet));
	buf1 = TPtrC16(name, TSTR_LEN(name));
	edwin->SetTextL(&buf1);
	edwin->SetReadOnly(ETrue);
	
	edwin=STATIC_CAST(CEikEdwin*, Control(EDetailsStatus)); 
	status = iscConfigGetSubsSetStatus(appPtr->hSyncConf, id);
	switch (status)
	{
		case ISCRTN_Succeeded :
			TSTR_CPY(statusBuf, ISYNCSTR_Succeeded);
			break;
		case ISCRTN_Canceled:
			TSTR_CPY(statusBuf, ISYNCSTR_Failed);
			break;
		case ISCRTN_Failed:
			TSTR_CPY(statusBuf, ISYNCSTR_Cancelled);
			break;
		case ISCRTN_Ready:
			TSTR_CPY(statusBuf, ISYNCSTR_Ready);
			break;
		default:
			TSTR_CPY(statusBuf, isy_T(""));
			break;			
	}

	buf2 = TPtrC16(statusBuf, TSTR_LEN(statusBuf));
	edwin->SetTextL(&buf2);
	edwin->SetReadOnly(ETrue);
}

TBool CDetailsDialog::OkToExitL(TInt aKeycode)
{
	EPOCGLOBALS* globals = epocGetGlobals();
	APPDATA* appPtr = (APPDATA*) globals->hApp;

	if (aKeycode == EEikBidOk)
	{
		//iCheckboxState = CheckBoxState(EDetailsResetCheckBox);
		CEikCheckBox* checkBox=static_cast<CEikCheckBox*>(Control(EDetailsResetCheckBox));
		iCheckboxState = checkBox->State();
		
		if(iCheckboxState == CEikButtonBase::ESet)
		{
			iscConfigResetSubsSet(appPtr->hSyncConf, id);
		}
	}
	return ETrue;
}

//======================================================================
// CJobsDialog

CJobsDialog::CJobsDialog(CModel* inModelPtr)
{
	syncStartup(NULL);
	iJobsPtr=&inModelPtr->Jobs();
	iJobsPtr->GetJobs();
}

// This function is called just before the dialog is displayed. It allows the
// dialog to be initialised with application data.
void CJobsDialog::PreLayoutDynInitL()
{
	CEikColumnListBox*		boxPtr=static_cast<CEikColumnListBox*>(Control(EJobList));

	// Add scroll bars to the list box.
	boxPtr->CreateScrollBarFrameL();
	boxPtr->ScrollBarFrame()->SetScrollBarVisibilityL(
		CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto
	);

	boxPtr->HandleItemAdditionL();

	BuildJobList();
	UpdatePopup();

	// Rename the cancel button to an ok button.
	TBuf<ISYNCLEN_MiscInfo> buttonName;
	CEikonEnv::Static()->ReadResource(buttonName, R_ISYNC_BUTTONNAME_OK);
	CEikCommandButton* buttonPtr = STATIC_CAST(CEikCommandButton*, ButtonGroupContainer().ButtonById(EEikBidCancel)) ;
	buttonPtr->SetTextL(buttonName);
	buttonPtr->DrawNow();
}

void CJobsDialog::PostLayoutDynInitL()
{
	CEikColumnListBox*		boxPtr=static_cast<CEikColumnListBox*>(Control(EJobList));
	CColumnListBoxItemDrawer* BoxItemDrawerPr=boxPtr->ItemDrawer();
	CColumnListBoxData* columnDataPtr=BoxItemDrawerPr->ColumnData();

	// Configure the columns. It seems that there is no explicit way to
	// add columns, one simply sets the size of the desired columns.
	const CFont* font = iEikonEnv->NormalFont();
	TRect bounds=boxPtr->Rect();
	columnDataPtr->SetColumnFontL(0, font);
	columnDataPtr->SetColumnWidthPixelL(0, TInt(bounds.Width()*0.6));
	columnDataPtr->SetColumnAlignmentL(0, CGraphicsContext::ELeft);
	columnDataPtr->SetColumnFontL(1, font);
	columnDataPtr->SetColumnWidthPixelL(1, TInt(bounds.Width()*0.4));
	columnDataPtr->SetColumnAlignmentL(1, CGraphicsContext::ELeft);
}

// This function is called whenever a button is pressed in the dialog's button
// panel (or a button is activated using its hotkey). It is not called for
// the Cancel button.
TBool CJobsDialog::OkToExitL(TInt aKeycode)
{
	switch (aKeycode)
	{
		case EJobsResetConfigButton:
		{
			CEikDialog* dialog = new (ELeave) CResetConfigDialog(ISYNCSTR_ResetConfig);
			dialog->ExecuteLD(R_MESSAGEBOX_CANCEL_OK_DIALOG);
			return EFalse;
		}
		case EJobsChangeButton:
			TryChangeFocusToL(EJobActionPopup);
			return EFalse;
		case EJobsDetailsButton:
		{
			CEikColumnListBox* boxPtr=static_cast<CEikColumnListBox*>(Control(EJobList));
			if(boxPtr->CurrentItemIndex() >= 0)
			{
				CEikDialog* dialog = new (ELeave) CDetailsDialog(boxPtr->CurrentItemIndex());
				dialog->ExecuteLD(R_DETAILS_DIALOG);
			}
			return EFalse;
		}
		default:
			break;
	}
	syncShutdown();
	return ETrue;
}

void CJobsDialog::HandleControlStateChangeL(TInt aControlId)
{
	switch(aControlId){
	case EJobList:
		UpdatePopup();
		break;
	case EJobActionPopup:
		UpdateJob();
		break;
	}
}

// Populate the list of jobs.
void CJobsDialog::BuildJobList()
{
	CEikColumnListBox*		boxPtr=static_cast<CEikColumnListBox*>(Control(EJobList));

	CTextListBoxModel*	  boxModelPr=boxPtr->Model();
 	MDesCArray* itemListPtr=	boxModelPr->ItemTextArray();
	CDesCArray* itemArrayPtr=	(CDesCArray*)itemListPtr;
	
	// To get content in each column we place a column seperator character
	// between the two sections of the string.

	itemArrayPtr->Reset();
	for(TInt i=0; i<iJobsPtr->ItemCount(); i++)
	{
		HBufC* row = iJobsPtr->NameDesForItem(i);   
		TBuf<ISYNCLEN_MiscInfo> row1 = row->Ptr();
		row1.SetLength(row->Length());
		row1.Append(KColumnListSeparator);
		row1+=iJobsPtr->ActionDesForItem(i);		
		itemArrayPtr->AppendL(row1);
		delete row;
	}
	boxPtr->HandleItemAdditionL();
	boxPtr->ActivateL();
	boxPtr->DrawNow();
}

// Updates the popup to reflect the action for the selected item.
void CJobsDialog::UpdatePopup()
{
	CEikColumnListBox* boxPtr=static_cast<CEikColumnListBox*>(Control(EJobList));
	
	if(boxPtr->CurrentItemIndex()>=0)
	{
		TInt actionIndex=iJobsPtr->ActionForItem(boxPtr->CurrentItemIndex());
		STATIC_CAST(CEikChoiceList*,Control(EJobActionPopup))->SetCurrentItem(actionIndex);
	}
}

// Updates the item to reflect a change in the action chosen in the popup.
void CJobsDialog::UpdateJob()
{
	// << change the options settings >>
	CEikColumnListBox* boxPtr = static_cast<CEikColumnListBox*>(Control(EJobList));

	BuildJobList();
	TInt listIndex(STATIC_CAST(CEikChoiceList*,Control(EJobActionPopup))->CurrentItem());

	CJobs::ESynchronizeAction action =  CJobs::synchronizeAction_Refresh;

	switch(listIndex) {
	case 0:
		action = CJobs::synchronizeAction_Skip;
		break;
	case 1:
		action = CJobs::synchronizeAction_Synchronize;
		break;
	case 2:
		action = CJobs::synchronizeAction_Refresh;
		break;
	case 3:
		action = CJobs::synchronizeAction_Resume;
		break;
	}

	iJobsPtr->SetActionForItem(boxPtr->CurrentItemIndex(), action);

	BuildJobList();
}

//======================================================================
// CProgressDialog
CProgressDialog::CProgressDialog()
{
	iPeriodicPtr=0;
	iContinueFlag = 0;
	messageBox = 0;
}

CProgressDialog::~CProgressDialog()
{
	delete iPeriodicPtr;
	delete iHandlerPtr;
}

// This function is called just before the dialog is displayed. It allows the
// dialog to be initialised with application data - in this case, the current
// co-ordinates stored in the model.
void CProgressDialog::PreLayoutDynInitL()
{

	TBuf16<ISYNCLEN_MiscInfo> buf;
	isy_TCHAR title[ISYNCLEN_MiscInfo];

	TSTR_CPY(title, isy_T("            "));
	TSTR_CAT(title, ISYNCSTR_Synchronization);
	TSTR_CAT(title, isy_T("            "));

	buf = TPtrC16(title, TSTR_LEN(title));
	SetTitleL(buf);

	// Make the critical section, aka semaphore, to protect the message buffer.
	TInt error=iMessageSem.CreateLocal();
	User::LeaveIfError(error);
	
	// Start the sync thread.
	iHandlerPtr = new (ELeave) CSynchronizationThreadHandler(this);

	EPOCGLOBALS* globals = epocGetGlobals();
	
	ISYNCPREFS prefs;
	appGetPref(ISYNCPREF_User, prefs.user, sizeof(prefs.user)/sizeof(isy_TCHAR)); // user ID

	while (TSTR_LEN(prefs.user) == 0) 
	{
		//pop up alert dialog box
		CEikDialog* dialog=new(ELeave) CMessageBoxWaitDialog(ISYNCSTR_NotSet);
		dialog->ExecuteLD(R_MESSAGEBOX_CONT_DIALOG_WAIT);
	
		//pop up settings dialog box
		dialog=new(ELeave) CServerSettingsDialog();
		dialog->ExecuteLD(R_SETTINGS_DIALOG);
		appGetPref(ISYNCPREF_User, prefs.user, sizeof(prefs.user)/sizeof(isy_TCHAR)); // user ID
	}

	iHandlerPtr->Synchronize(globals);

	// Start the pulse. The the comment above Tick().
	iPeriodicPtr=CPeriodic::NewL(EPriorityMuchLess);
	iPeriodicPtr->Start(100000, 100000, TCallBack(UpdateMessage, this));
}

// This function is called whenever a button is pressed in the dialog's button
// panel (or a button is activated using its hotkey). It is not normally
// called for the Cancel button, but because of the EEikDialogFlagNotifyEsc
// flag it is.
TBool CProgressDialog::OkToExitL(TInt /*aKeycode*/)
{
	EPOCGLOBALS* globals = epocGetGlobals();
	EPUIDATA* dataPtr = (EPUIDATA*) globals->hData;

	if (dataPtr->cancelPressed == 0) {
		//cancel case
		dataPtr->cancelPressed = 1;
		PostMessageL(ISYNCSTR_InfCancelingSync);	
		DrawNow();

		return EFalse;
	}
	else if (dataPtr->cancelPressed == 2) 
	{
		return ETrue;
	}
	else 
		return EFalse;
}

void CProgressDialog::PostMessageL(const TBuf16<ISYNCLEN_MiscInfo>& inMessage)
{
 	iMessageSem.Wait();
	iMessage.Copy(inMessage);
	iMessageSem.Signal();
}

void CProgressDialog::PostMessageErrorL(const TBuf16<ISYNCLEN_MiscInfo>& inMessage)
{
  
	iMessageSem.Wait();
	iMessage.Copy(inMessage);
	iMessageSem.Signal();
	
	//set internal flag to indicate that the button is now "Continue"
	iContinueFlag = 1;

	//change button name to "Cont"

	TBuf<ISYNCLEN_MiscInfo> buffer;
	buffer = TPtrC16(ISYNCSTR_Continue);
	CEikCommandButton* cancelButtonPtr=STATIC_CAST(CEikCommandButton*, ButtonGroupContainer().ButtonById(EEikBidCancel));
		cancelButtonPtr->SetTextL(buffer);
	cancelButtonPtr->DrawDeferred();

	User::Beep(440, 100);

	//wait until user clicks button to continue this thread
	for (;;)
	{
		if (iContinueFlag == 0) 
			break;
	}
}

//this function is called by periodic function
TInt CProgressDialog::UpdateMessage(TAny* inProgressDialogPtr)
{
	CProgressDialog* dialogPtr=(CProgressDialog*)inProgressDialogPtr;
	dialogPtr->iMessageSem.Wait();

	//appMessageBox() or dataPtr->messageBox was called in listener
	if (dialogPtr->messageBox)
	{
		if (dialogPtr->messageBox == APPMB_LOGIN)
		{
			dialogPtr->iPeriodicPtr->Cancel();
			CEikDialog* dialog = new (ELeave) CLoginDialog((ISCLISTENARG*) dialogPtr->args);
			dialog->ExecuteLD(R_LOGIN_DIALOG);
			dialogPtr->iPeriodicPtr->Start(100000, 100000, TCallBack(UpdateMessage, dialogPtr));
		}
		else
		{
			CEikDialog* dialog = new (ELeave) CMessageBoxDialog(dialogPtr->message);
			if (dialogPtr->messageBox == APPMB_OKCANCEL)
				dialog->ExecuteLD(R_MESSAGEBOX_CANCEL_OK_DIALOG);
			else if(dialogPtr->messageBox == APPMB_YESNO)
				dialog->ExecuteLD(R_MESSAGEBOX_NO_YES_DIALOG);
			else
				dialog->ExecuteLD(R_MESSAGEBOX_CONT_DIALOG);

		}

		dialogPtr->messageBox = 0;
		dialogPtr->iMessageSem.Signal();

	}
	//else update progress label
	else 
	{
		TBuf<ISYNCLEN_MiscInfo> buffer;
		buffer = TPtrC16(dialogPtr->iMessage);
		CEikLabel* aLabel= STATIC_CAST(CEikLabel*, dialogPtr->Control(EProgressLabel1));
		aLabel->SetTextL(buffer);
		aLabel->DrawNow();
		dialogPtr->iMessageSem.Signal();
	}

	return 0;
}


void CProgressDialog::WorkerThreadFinishedL()
{

	TBuf<ISYNCLEN_MiscInfo> buttonName;
	CEikonEnv::Static()->ReadResource(buttonName, R_ISYNC_BUTTONNAME_OK);
	CEikCommandButton* cancelButtonPtr=STATIC_CAST(CEikCommandButton*, ButtonGroupContainer().ButtonById(EEikBidCancel));
	cancelButtonPtr->SetTextL(buttonName);
	cancelButtonPtr->DrawNow();

	EPOCGLOBALS* globals = epocGetGlobals();
	EPUIDATA* dataPtr = (EPUIDATA*) globals->hData;
	
	dataPtr->cancelPressed = 2;
}

//======================================================================
// CLogDialog
CLogDialog::CLogDialog()
{
}

CLogDialog::~CLogDialog()
{
}

// This function is called just before the dialog is displayed. It allows the
// dialog to be initialised with application data - in this case, the current
// co-ordinates stored in the model.
void CLogDialog::PreLayoutDynInitL()
{
	CEikEdwin* edwinPtr=STATIC_CAST(CEikEdwin*, Control(ELogTextArea));
	edwinPtr->CreateTextViewL();
	edwinPtr->SetRightWrapGutter(10);

	isy_TCHAR fileName[ISYNCLEN_MiscInfo];
	TSTR_CPY(fileName, DATADIR);
	TSTR_CAT(fileName, appLogDbName);

	TBuf16<ISYNCLEN_MiscInfo> des;
	CharPtrToFieldDes(fileName, des);
	edwinPtr->InsertFromTextFileL(des, CPlainText::EOrganiseByParagraph);
  
	// Do the scroll bars.
	edwinPtr->CreateScrollBarFrameL();
	edwinPtr->ScrollBarFrame()->SetScrollBarVisibilityL
	(
		CEikScrollBarFrame::EAuto, CEikScrollBarFrame::EAuto
	);
	
	// Rename the cancel button to an ok button.
	TBuf<ISYNCLEN_MiscInfo> buttonName;
	CEikonEnv::Static()->ReadResource(buttonName, R_ISYNC_BUTTONNAME_OK);
	CEikCommandButton* buttonPtr=STATIC_CAST(CEikCommandButton*,ButtonGroupContainer().ButtonById(EEikBidCancel));
	buttonPtr->SetTextL(buttonName);
	buttonPtr->DrawNow();

}

// This function is called whenever a button is pressed in the dialog's button
// panel (or a button is activated using its hotkey).
TBool CLogDialog::OkToExitL(TInt /*aKeycode*/)
{
	//OkToExitL not called since button is actually Cancel button
	return ETrue;
}

//======================================================================
// CAboutDialog
CAboutDialog::CAboutDialog()
{
	syncStartup(NULL);
}

CAboutDialog::~CAboutDialog()
{
	syncShutdown();
}

// This function is called just before the dialog is displayed. It allows the
// dialog to be initialised with application data - in this case, the current
// co-ordinates stored in the model.
void CAboutDialog::PreLayoutDynInitL()
{
	EPOCGLOBALS* globals = epocGetGlobals();
	APPDATA* appPtr = (APPDATA*) globals->hApp;
	isy_TCHAR about[ISYNCLEN_MiscInfo];

	CEikEdwin* edwinPtr;

	//about box
	if (iscEngineGetInfo(appPtr->hSyncer, about, ISYNCLEN_MiscInfo) == ISCRTN_Succeeded)
	{
		TBuf16<ISYNCLEN_MiscInfo> buf1;
		edwinPtr=STATIC_CAST(CEikEdwin*, Control(EAboutTextArea));
		buf1 = TPtrC16(about, TSTR_LEN(about));
		edwinPtr->SetTextL(&buf1);
		// Do the scroll bars.
		edwinPtr->CreateScrollBarFrameL();
		edwinPtr->ScrollBarFrame()->SetScrollBarVisibilityL
		(
			CEikScrollBarFrame::EAuto, CEikScrollBarFrame::EAuto
		);

	}

}

TBool CAboutDialog::OkToExitL(TInt /*aKeycode*/)
{
	return ETrue;
}

//======================================================================
// CMessageBoxDialog
CMessageBoxDialog::CMessageBoxDialog(const isy_TCHAR* msg)
{
	message = (isy_TCHAR *)msg;
}

CMessageBoxDialog::~CMessageBoxDialog()
{
}

// This function is called just before the dialog is displayed. It allows the
// dialog to be initialised with application data - in this case, the current
// co-ordinates stored in the model.
void CMessageBoxDialog::PreLayoutDynInitL()
{

	CEikEdwin* edwinPtr=STATIC_CAST(CEikEdwin*, Control(ELogTextArea));
	// Do the scroll bars.
	edwinPtr->CreateScrollBarFrameL();
	edwinPtr->ScrollBarFrame()->SetScrollBarVisibilityL(
		CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto
	);

	TBuf16<ISYNCLEN_MiscInfo> buf1 = TPtrC16(&message[0], TSTR_LEN(message));
	edwinPtr->SetTextL(&buf1);
}

// This function is called whenever a button is pressed in the dialog's button
// panel (or a button is activated using its hotkey).
TBool CMessageBoxDialog::OkToExitL(TInt aKeycode)
{
	EPOCGLOBALS* globals = epocGetGlobals();
	EPUIDATA* dataPtr = (EPUIDATA*) globals->hData;
	
	if (aKeycode == EEikBidOk)
		dataPtr->msgBoxButtonClicked = MessageOKButton;

	if (aKeycode == EEikBidCancel)
		dataPtr->msgBoxButtonClicked = MessageCancelButton;

	else
		dataPtr->msgBoxButtonClicked = MessageOKButton;
		
	return ETrue;
}

//======================================================================
// CResetConfigDialog
CResetConfigDialog::CResetConfigDialog(const isy_TCHAR* msg)
{
	message = (isy_TCHAR *)msg;
}

CResetConfigDialog::~CResetConfigDialog()
{
}

// This function is called just before the dialog is displayed. It allows the
// dialog to be initialised with application data - in this case, the current
// co-ordinates stored in the model.
void CResetConfigDialog::PreLayoutDynInitL()
{
	CEikEdwin* edwinPtr=STATIC_CAST(CEikEdwin*, Control(ELogTextArea));
	// Do the scroll bars.
	edwinPtr->CreateScrollBarFrameL();
	edwinPtr->ScrollBarFrame()->SetScrollBarVisibilityL(
		CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto
	);

	TBuf16<ISYNCLEN_MiscInfo> buf1 = TPtrC16(&message[0], TSTR_LEN(message));
	edwinPtr->SetTextL(&buf1);
}

// This function is called whenever a button is pressed in the dialog's button
// panel (or a button is activated using its hotkey).
TBool CResetConfigDialog::OkToExitL(TInt aKeycode)
{
	EPOCGLOBALS* globals = epocGetGlobals();
	APPDATA* appPtr = (APPDATA*) globals->hApp;
	
	if (aKeycode == EEikBidOk)
	{
		iscConfigPurge(appPtr->hSyncConf);
	}
	return ETrue;
}

//======================================================================
// CMessageBoxWaitDialog
CMessageBoxWaitDialog::CMessageBoxWaitDialog(const isy_TCHAR* msg)
{
	message = (isy_TCHAR*)msg;
}

CMessageBoxWaitDialog::~CMessageBoxWaitDialog()
{
}

// This function is called just before the dialog is displayed. It allows the
// dialog to be initialised with application data - in this case, the current
// co-ordinates stored in the model.
void CMessageBoxWaitDialog::PreLayoutDynInitL()
{
	CEikEdwin* edwinPtr=STATIC_CAST(CEikEdwin*, Control(ELogTextArea));

	// Do the scroll bars.
	edwinPtr->CreateScrollBarFrameL();
	edwinPtr->ScrollBarFrame()->SetScrollBarVisibilityL(
		CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto
	);

	TBuf16<ISYNCLEN_MiscInfo> buf1 = TPtrC16(&message[0], TSTR_LEN(message));
	edwinPtr->SetTextL(&buf1);
}

// This function is called whenever a button is pressed in the dialog's button
// panel (or a button is activated using its hotkey).
TBool CMessageBoxWaitDialog::OkToExitL(TInt /*aKeycode*/)
{		
	return ETrue;
}
