/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  Dialogs.h
 *
 * Description:
 *    Classes for implementing dialog boxes
 *
 */

#ifndef _DIALOGS_H
#define _DIALOGS_H

#include "Model.h"
#include "Workerthread.h"

#include <eikdialg.h>
#include <eikenv.h>
#include <eikon.rsg>
#include <eikdialg.h>
#include <eikclb.h>
#include <eiktxlbm.h>
#include <eikedwin.h>
#include <eikseced.h>
#include <eikchkbx.h> 
#include <eiklabel.h>
#include <gulutil.h>

const TInt KHorzEdgeSpace = 6;

//======================================================================
// CLoginDialog
class CLoginDialog : public CEikDialog
{
public:
	CLoginDialog(ISCLISTENARG *args);
	~CLoginDialog();

private:
	void PreLayoutDynInitL();
	TBool OkToExitL(TInt aKeycode);
#ifdef EPV7_QZ
	TBool ShutL() { return EFalse; }
#endif	
	
	ISCLISTENARG *args;
};

//======================================================================
// CServerSettingsDialog
class CServerSettingsDialog : public CEikDialog
{
public:
	CServerSettingsDialog();
	~CServerSettingsDialog();

private:
	void PreLayoutDynInitL();
	TBool OkToExitL(TInt aKeycode);
#ifdef EPV7_QZ
	TBool ShutL() { return EFalse; }
#endif	
	
	CEikButtonBase::TState iCheckboxState;
	CEikSecretEditor* iSecretEditor;
};

//======================================================================
// CClientSettingsDialog
class CClientSettingsDialog : public CEikDialog
{
public:
	CClientSettingsDialog();
	~CClientSettingsDialog();

private:
	void PreLayoutDynInitL();
	TBool OkToExitL(TInt aKeycode);
#ifdef EPV7_QZ
	TBool ShutL() { return EFalse; }
#endif	
	
	CEikButtonBase::TState iCheckboxState;
};

//======================================================================
// CPasswordDialog
class CPasswordDialog : public CEikDialog
{
public:
	CPasswordDialog();
	~CPasswordDialog();

private:
	CEikButtonBase::TState iCheckboxState;
	CEikCheckBox* iCheckBox;
	void  PreLayoutDynInitL();
	TBool OkToExitL(TInt);
#ifdef EPV7_QZ
	TBool ShutL() { return EFalse; }
#endif	
	
	CEikSecretEditor* iSecretEditor;
};

//======================================================================
// CNetworkSettingsDialog
class CNetworkSettingsDialog : public CEikDialog
{
public:
	CNetworkSettingsDialog();
	~CNetworkSettingsDialog();

private:
	CEikButtonBase::TState iCheckboxState;
	void  PreLayoutDynInitL();
	TBool OkToExitL(TInt);
#ifdef EPV7_QZ
	TBool ShutL() { return EFalse; }
#endif	
	
	virtual void	HandleControlStateChangeL(TInt aControlId);
};

//======================================================================
// CDetailsDialog
class CDetailsDialog : public CEikDialog
{
public:
	CDetailsDialog(TInt index);
	~CDetailsDialog();

private:
	CEikButtonBase::TState iCheckboxState;
	void	PreLayoutDynInitL();
	TBool	OkToExitL(TInt aKeycode);
#ifdef EPV7_QZ
	TBool ShutL() { return EFalse; }
#endif	
	
	isy_TCHAR id[ISCLEN_SubsSetID], name[ISCLEN_SubsSetName];
	TInt listIndex;
};

//======================================================================
// CJobsDialog
class CJobsDialog : public CEikDialog
{
public:
	CJobsDialog(CModel* inModelPtr);

private:
	virtual void	PreLayoutDynInitL();
	virtual void	PostLayoutDynInitL();
	virtual TBool	OkToExitL(TInt aKeycode);
#ifdef EPV7_QZ
	TBool ShutL() { return EFalse; }
#endif	
	
	virtual void	HandleControlStateChangeL(TInt aControlId);

	void	BuildJobList();
	void	UpdatePopup();
	void	UpdateJob();

	CJobs*	iJobsPtr;
};

//======================================================================
// CProgressDialog
class CProgressDialog:public CEikDialog
{
public:
	CProgressDialog();
	virtual ~CProgressDialog();

	TBuf16<ISYNCLEN_MiscInfo>	iMessage;
	void	PostMessageL(const TBuf16<ISYNCLEN_MiscInfo>& inMessage);
	void	PostMessageErrorL(const TBuf16<ISYNCLEN_MiscInfo>& inMessage);
	void	WorkerThreadFinishedL();	// Called by CSynchronizationThreadHandler::RunL().

	void	SetMessage(const isy_TCHAR* msg)	{TSTR_CPY(message, msg);};
	void	SetMessageBoxFlag(int flag)			{messageBox = flag;};
	void	SetArgs(void *args)					{this->args = args;};

	//messagebox button flag
	int msgBoxButtonClicked;

private:
	virtual	void	PreLayoutDynInitL();
	virtual	TBool	OkToExitL(TInt aKeycode);
#ifdef EPV7_QZ
	TBool ShutL() { return EFalse; }
#endif	
	
	static TInt		Tick(TAny* inAnyPtr);
	static TInt		UpdateMessage(TAny* inProgressDialogPtr);
	int iContinueFlag;  // 0 = cancel, 1 = continue, 2 = cancel pressed, 3 = ok

	RCriticalSection				iMessageSem;

	CSynchronizationThreadHandler*	iHandlerPtr;
	CPeriodic*						iPeriodicPtr;
	//type of messageBox - APPMB_OK, APPMB_OKCANCEL, APPMB_YESNO, APPMB_LOGIN
	int messageBox;
	isy_TCHAR message[ISYNCLEN_MiscInfo];
	//login
	void* args;

};


//======================================================================
// CLogDialog
class CLogDialog : public CEikDialog
{
public:
	CLogDialog();
	virtual ~CLogDialog();

private:
	virtual	void	PreLayoutDynInitL();
	virtual	TBool	OkToExitL(TInt aKeycode);
#ifdef EPV7_QZ
	TBool ShutL() { return EFalse; }
#endif	
	
};

//======================================================================
// CAboutDialog
class CAboutDialog : public CEikDialog
{
public:
	CAboutDialog();
	virtual ~CAboutDialog();

private:
	virtual	void	PreLayoutDynInitL();
	virtual	TBool	OkToExitL(TInt aKeycode);
#ifdef EPV7_QZ
	TBool ShutL() { return EFalse; }
#endif	
	
};

//======================================================================
// CMessageBoxDialog
class CMessageBoxDialog:public CEikDialog
{
public:
	CMessageBoxDialog(const isy_TCHAR* message);
	virtual ~CMessageBoxDialog();

private:
	virtual	void	PreLayoutDynInitL();
	virtual	TBool	OkToExitL(TInt aKeycode);
#ifdef EPV7_QZ
	TBool ShutL() { return EFalse; }
#endif	

	isy_TCHAR* message;
};

//======================================================================
// CResetConfigDialog
class CResetConfigDialog:public CEikDialog
{
public:
	CResetConfigDialog(const isy_TCHAR* message);
	virtual ~CResetConfigDialog();

private:
	virtual	void	PreLayoutDynInitL();
	virtual	TBool	OkToExitL(TInt aKeycode);
#ifdef EPV7_QZ
	TBool ShutL() { return EFalse; }
#endif	

	isy_TCHAR* message;
};

//======================================================================
// CMessageBoxWaitDialog
class CMessageBoxWaitDialog:public CEikDialog
{
public:
	CMessageBoxWaitDialog(const isy_TCHAR* message);
	virtual ~CMessageBoxWaitDialog();

private:
	virtual	void	PreLayoutDynInitL();
	virtual	TBool	OkToExitL(TInt /*aKeycode*/);
#ifdef EPV7_QZ
	TBool ShutL() { return EFalse; }
#endif	

	isy_TCHAR* message;
};

#endif //_DIALOGS_H