/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  Model.cpp
 *
 * Description:
 *    Classes for storing user-input information and utility routines
 *
 */

#include "Model.h"

//obsolete
#define ISYNCMODE_Skip                          0x0000
#define ISYNCMODE_Resume                        0x0004

//======================================================================
void ConvertUnicodeTextL(CCnvCharacterSetConverter& aCharacterSetConverter, 
            RFs& aFileServerSession, TUint aForeignCharacterSet, 
            const TDesC16& aUnicodeText, TDes8& outputBuffer)
    {
		// check to see if the character set is supported - if not then leave 
		if (aCharacterSetConverter.PrepareToConvertToOrFromL(aForeignCharacterSet, 
                aFileServerSession) != CCnvCharacterSetConverter::EAvailable)
			User::Leave(KErrNotSupported);
    
		// Create a buffer for the unconverted text - initialised with input descriptor
		TPtrC16 remainderOfUnicodeText(aUnicodeText); 
		for(;;) // conversion loop
        {
			// Start conversion. When the output buffer is full, return the 
			// number of characters that were not converted
			 TInt failedCount = 0;
			const TInt returnValue=aCharacterSetConverter.ConvertFromUnicode(outputBuffer, 
								   remainderOfUnicodeText, failedCount);
			if(failedCount != 0)
			   User::Leave(KErrCorrupt);
			// check to see that the descriptor isnt corrupt - leave if it is
			if (returnValue==CCnvCharacterSetConverter::EErrorIllFormedInput)
				User::Leave(KErrCorrupt);
			else if (returnValue<0) // future-proof against "TError" expanding
				User::Leave(KErrGeneral);
			// Do something here to store the contents of the output buffer. 
			// Finish conversion if there are no unconverted characters in the remainder buffer
			if (returnValue==0)
				break; 
			// Remove the converted source text from the remainder buffer. 
			// The remainder buffer is then fed back into loop
			remainderOfUnicodeText.Set(remainderOfUnicodeText.Right(returnValue));
        }
    }

//========================================================================
void ConvertForeignTextL(CCnvCharacterSetConverter& aCharacterSetConverter,
            RFs& aFileServerSession, TUint aForeignCharacterSet, 
            const TDesC8& aForeignText, TDes16& outputBuffer)
   {
		// check to see if the character set is supported 
		// - if not then leave
		if (aCharacterSetConverter.PrepareToConvertToOrFromL(aForeignCharacterSet, 
						aFileServerSession) != CCnvCharacterSetConverter::EAvailable)
			User::Leave(KErrNotSupported);
		// Create a buffer for the unconverted text - initialised with the input descriptor
		TPtrC8 remainderOfForeignText(aForeignText);
		// Create a "state" variable and initialise it with CCnvCharacterSetConverter::KStateDefault
		// After initialisation the state variable must not be tampered with.
		// Simply pass into each subsequent call of ConvertToUnicode()
		TInt state=CCnvCharacterSetConverter::KStateDefault;
		for(;;) // conversion loop
        {
			// Start conversion. When the output buffer is full, return the number
			// of characters that were not converted
			TInt failedCount = 0;
			const TInt returnValue=aCharacterSetConverter.ConvertToUnicode(outputBuffer, 
					   remainderOfForeignText, state, failedCount);
        
			if(failedCount != 0)
			   User::Leave(KErrCorrupt); 
			// check to see that the descriptor isnt corrupt - leave if it is
			if (returnValue==CCnvCharacterSetConverter::EErrorIllFormedInput)
				User::Leave(KErrCorrupt);
			else if (returnValue<0) // future-proof against "TError" expanding
				User::Leave(KErrGeneral);
			// Do something here to store the contents of the output buffer.
			// Finish conversion if there are no unconverted 
			// characters in the remainder buffer
			if (returnValue==0)
				break;
			// Remove converted source text from the remainder buffer.
			// The remainder buffer is then fed back into loop
			remainderOfForeignText.Set(remainderOfForeignText.Right(returnValue));
        }
    }
 
//======================================================================
// Utilities.
void DesCToCharPtr(HBufC* pinDes, isy_TCHAR* inPtr)
{
	if(pinDes)
	{
#ifndef _UNICODE
		const TUint8* desBufPtr=pinDes->Ptr();
		TInt size=pinDes->Size();
		for(TInt i=0; i<size/2; i++)
		{
			// Crudely convert 16-bit chars to 8-bit chars by taking the
			// low-order byte.
			inPtr[i]= desBufPtr[i];
		}
		inPtr[i] = '\0'; //zero terminate it
#else
		const TUint16* desBufPtr=pinDes->Ptr();
		TInt size=pinDes->Size()/2;
		TInt i=0;
		for(; i<size; i++)
		{
		   inPtr[i]=desBufPtr[i];
		}
		inPtr[size]= isy_TCHAR('\0');
#endif
	}
	else
		MEM_SET(inPtr, 0, sizeof(inPtr)); 
}

void CharPtrToFieldDes(const isy_TCHAR* inPtr, TBuf16<ISYNCLEN_MiscInfo>& outDes)
{
	outDes.Format(_L("%s"), inPtr);
	outDes.SetLength(TSTR_LEN(inPtr));
}

void CharPtrToTIntArray(const char* inPtr, TUint8* array, TInt len)
{
	TInt i = 0;
	for(i=0;i<len; i++)  //clean the output array
		array[i] = 0;
	
	for(i=0 ; i < len; i++)
	{
		array[i] = inPtr[i];
		if(inPtr[i] == '\0') //assume the string ends with '\0'
			break;
	}
	array[i] = '\0';
}


//======================================================================
// CModel
CModel::CModel()
{
}

CModel::~CModel()
{
}

void CModel::ConstructL()
{
}

//======================================================================
// CJobs
CJobs::CJobs()
{
	CEikonEnv::Static()->ReadResource(iActionName_Skip,			R_ACTIONSTRING_SKIP);
	CEikonEnv::Static()->ReadResource(iActionName_Synchronize,	R_ACTIONSTRING_SYNCHRONIZE);
	CEikonEnv::Static()->ReadResource(iActionName_Refresh,		R_ACTIONSTRING_REFRESH);
	CEikonEnv::Static()->ReadResource(iActionName_Resume,		R_ACTIONSTRING_RESUME);

	size = 50;
	jobArray = (Jobs*) MEM_PTR_ALLOC(sizeof(Jobs) * size);
	numJobs = 0;
}

CJobs::~CJobs()
{
	MEM_PTR_FREE(jobArray);
}

TInt CJobs::ItemCount()
{
	return numJobs;
}

CJobs::ESynchronizeAction CJobs::ActionForItem(TInt inIndex)
{
	Jobs job = jobArray[inIndex];

	switch(job.mode) 
	{
		case ISYNCMODE_Skip:
			return synchronizeAction_Skip;
		case ISYNCMODE_Synchronize:
			return synchronizeAction_Synchronize;
		case ISYNCMODE_Refresh:
			return synchronizeAction_Refresh;
		default:
			return ESynchronizeAction(-1);
	}
}

TBufC<ISYNCLEN_MiscInfo> CJobs::ActionDesForItem(TInt inIndex)
{
	switch(ActionForItem(inIndex)){
	case synchronizeAction_Skip:		return iActionName_Skip;
	case synchronizeAction_Synchronize:	return iActionName_Synchronize;
	case synchronizeAction_Refresh:		return iActionName_Refresh;
	case synchronizeAction_Resume:		return iActionName_Resume;
	default:							return _L("<< bad action >>");
	}
}

void CJobs::SetActionForItem(TInt inIndex, ESynchronizeAction inAction)
{
	isy_TCHAR id[ISCLEN_SubsSetID], name[ISCLEN_SubsSetName];

	EPOCGLOBALS* globals = epocGetGlobals();
	APPDATA* appPtr = (APPDATA*) globals->hApp;

	HISCCSR	hCursor;
	int rc = ISCRTN_Succeeded;
	
	switch(inAction){
	case synchronizeAction_Skip:
		jobArray[inIndex].mode = ISYNCMODE_Skip;
		break;
	case synchronizeAction_Synchronize:
		jobArray[inIndex].mode = ISYNCMODE_Synchronize;
		break;
	case synchronizeAction_Refresh:
		jobArray[inIndex].mode = ISYNCMODE_Refresh;
		break;
	default:
		break;
	}


	iscConfigOpenCursor(appPtr->hSyncConf, &hCursor);
	//inIndex is the selected app in jobslist
	for (int i=0; i <= inIndex; i++)
	{
		rc = iscConfigGetNextSubsSet(appPtr->hSyncConf, hCursor, id, name);
		if (rc != ISCRTN_Succeeded) 
			break;
	}
	iscConfigCloseCursor(appPtr->hSyncConf, hCursor);

	if (rc == ISCRTN_Succeeded) 
	{
		if(inAction == synchronizeAction_Synchronize)
			iscConfigEnableSubsSet(appPtr->hSyncConf, id);
		else if (inAction == synchronizeAction_Refresh)
			iscConfigResetSubsSet(appPtr->hSyncConf, id);
		else if (inAction == synchronizeAction_Skip)
			iscConfigDisableSubsSet(appPtr->hSyncConf, id);
	}
}

HBufC* CJobs::NameDesForItem(TInt inIndex)
{
	int len;
	len = TSTR_LEN(jobArray[inIndex].name);

	TBuf16<ISCLEN_SubsSetName> buf1 = TPtrC(&jobArray[inIndex].name[0], len);
	HBufC* poutDes = buf1.AllocL();
	return poutDes;
}


void CJobs::GetJobs()
{
	EPOCGLOBALS* globals = epocGetGlobals();
	APPDATA* appPtr = (APPDATA*) globals->hApp;
	isy_TCHAR id[ISCLEN_SubsSetID], name[ISCLEN_SubsSetName];
	int index = 0;

	HISCCSR	hCursor;
	iscConfigOpenCursor(appPtr->hSyncConf, &hCursor);
	while( iscConfigGetNextSubsSet(appPtr->hSyncConf, hCursor, id, name) == ISCRTN_Succeeded)
	{
		TSTR_CPY(jobArray[index].name, name);

		if (iscConfigSubsSetIsEnabled(appPtr->hSyncConf, id))
				jobArray[index].mode = ISYNCMODE_Synchronize;
		else
				jobArray[index].mode = ISYNCMODE_Skip;

		index++;
		if (index == size) 
		{
			Jobs* tempJobArray = (Jobs*) MEM_PTR_ALLOC(sizeof(Jobs) * size * 2);

			for (int i=0; i<size; i++)
				tempJobArray[i] = jobArray[i];

			MEM_PTR_FREE(jobArray);
			jobArray = tempJobArray;
			size = size * 2;
		}
	}
	iscConfigCloseCursor(appPtr->hSyncConf, hCursor);
	numJobs = index;
}
