/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name: SyncDriver.c
 *
 * Description:
 *    top-level C functions of the syncengine
 *
 */

//#include <e32def.h>
//#include <stdio.h>

#include "SyncDriver.h"
#include "db2sync.hrh"

static isy_INT32 syncListener(isy_UINT32 listenerData, ISCEVT* event, isy_VOID* pExtraInfo);

void syncLoop()
{
	EPOCGLOBALS* globals = epocGetGlobals();
	APPDATA* appPtr = (APPDATA*) globals->hApp;
	EPUIDATA* dataPtr = (EPUIDATA*) globals->hData;

	isy_INT32 rc = 0;
	
	rc = syncStartup(syncListener);

	dataPtr->cancelPressed = 0;	
	appPtr->errorCount = 0;

	appPtr->hLog = logOpen(appPtr);
	logPurge(appPtr->hLog);
	appLogBuildVersion(appPtr);

	if (rc != ISCRTN_Succeeded)
	{
		isy_TCHAR statusMsg[128];
		TSTR_CPY(statusMsg, ISYNCSTR_StartupError);
		//rc should be ISCRTN_NotFound (-3)
		TSTR_ITOA(statusMsg + TSTR_LEN(statusMsg), rc);
		logWrite(appPtr->hLog, statusMsg);

		dataPtr->setStatus(ISYNCSTR_InfSyncFailed, dataPtr->engine);
		goto exit1;
	}

	rc = iscEngineSync(appPtr->hSyncer);

	logClose(appPtr->hLog);

	if (appPtr->sessionPassword[0])
	{
		ISYNCPREFS prefs;
		prefs.password[0] = '\0';
		appSetPref(ISYNCPREF_UnsavedPassword, prefs.password);
	}
exit1:
	syncShutdown();
}

//callback listener
static isy_INT32 syncListener(isy_UINT32 listenerData, ISCEVT* event, isy_VOID* pExtraInfo)
{
	EPOCGLOBALS* globals = epocGetGlobals();
	APPDATA* appPtr = (APPDATA*) globals->hApp;
	//dataPtr is pointer to Progress dialog
	EPUIDATA* dataPtr = (EPUIDATA*) globals->hData;

	isy_TCHAR buf[ISYNCLEN_MiscInfo];
	isy_TCHAR percent[4];
	isy_TCHAR *statusMsg = (isy_TCHAR*) appEventCodeToMessage(event->code);
	
	switch (event->type) {
		case ISCEVTTYPE_Retry:
		{
/*
			long timeout = 60;
			int timeoutReturnValue = ISCRTNCB_ReplyNo;
			int rc = appMessageBox2(statusMsg, APPMB_YESNO, timeout, timeoutReturnValue);
*/
			int rc = dataPtr->messageBox(statusMsg, dataPtr->engine, APPMB_YESNO, NULL);
			
			if(rc == MessageOKButton) //yes pressed
				return ISCRTNCB_ReplyYes;
			else 
				return ISCRTNCB_ReplyNo;				
		} 
		break;
		
        case ISCEVTTYPE_Conflict:
		{
			//from sqlcli.h
			//#define  SQL_INTEGER             4
			//#define  SQL_SMALLINT            5
			#define SQL_BLOB					-98

			ISCLISTENCONFLICT *pConflict = (ISCLISTENCONFLICT *) event->info;
			int i=0;
			if (!pConflict) return ISCRTNCB_Default;
			TSTR_CPY(buf, ISYNCSTR_MSG_CONFLICT);
			TSTR_CAT(buf, pConflict->table);
			appLogWrite(appPtr, event->code, event->type, buf);
			switch (pConflict->op)
			{
				case 1:
					TSTR_CPY(buf, ISYNCSTR_MSG_OP_DELETE);
					TSTR_CAT(buf, FILE_NL);
					TSTR_CAT(buf, ISYNCSTR_MSG_DATA);
					break;
				case 2:
					TSTR_CPY(buf, ISYNCSTR_MSG_OP_INSERT);
					TSTR_CAT(buf, FILE_NL);
					TSTR_CAT(buf, ISYNCSTR_MSG_DATA);
					break;
				case 3:
					TSTR_CPY(buf, ISYNCSTR_MSG_OP_UPDATE);
					TSTR_CAT(buf, FILE_NL);
					TSTR_CAT(buf, ISYNCSTR_MSG_DATA);
					break;
				default:
					TSTR_CPY(buf, ISYNCSTR_MSG_OP_INVALID);
					TSTR_CAT(buf, FILE_NL);
					TSTR_CAT(buf, ISYNCSTR_MSG_DATA);
					break;
			}
			logWrite(appPtr->hLog, buf);

			for (i=0; i<pConflict->colc; i++)
			{
				ISCLISTENCOLUMN *pColumn = &(pConflict->colv[i]);
				MEM_SET(buf, 0, sizeof(buf));

				if (pColumn->data == NULL) {
					TSTR_CAT(buf, ISYNCSTR_MSG_NULL);
				}
				else if (pColumn->type == SQL_BLOB) {
					TSTR_CAT(buf, isy_T(" [ * ] "));
				}
				else {
					isy_BYTE* pData;
					// for null terminated character
					pData = (isy_BYTE *) MEM_PTR_ALLOC(pColumn->size + 2);
					if (pData == NULL) break;
					MEM_CPY(pData, &(pColumn->data[0]), pColumn->size);
					pData[pColumn->size] = 0;
					MEM_CPY(buf, pData, pColumn->size > 100 ? 100:pColumn->size);
					MEM_PTR_FREE(pData);
				}
				logWrite(appPtr->hLog, buf);
			}
		} //end ISCEVTTYPE_Conflict
		break;
		case ISCEVTTYPE_Query:
		{
			switch (event->code) 
			{
				case ISCEVT_QueCancel:					
					if (dataPtr->cancelPressed)
						return ISCRTNCB_ReplyYes;
					else
						return ISCRTNCB_ReplyNo;
				break;
				case ISCEVT_QueCancelUponError:
				{	
/*
					long timeout = 60;
					int timeoutReturnValue = ISCRTNCB_ReplyNo;
					// Ask if continue with the next subscription set upon the occurrence of an error
					int rc = appMessageBox2(statusMsg, APPMB_YESNO, timeout, timeoutReturnValue);
*/
					int rc = dataPtr->messageBox(statusMsg, dataPtr->engine, APPMB_YESNO, NULL);
					
					if(rc == MessageOKButton) // OK pressed, continue
						return ISCRTNCB_ReplyNo; // Don't cancel
					else 
						return ISCRTNCB_ReplyYes; // Cancel
				}
				break;
				case ISCEVT_QueLogin:
				{
					ISCLISTENARG *args = event->info;
					
					//show login dialog
					dataPtr->messageBox(isy_T(""), dataPtr->engine, APPMB_LOGIN, args);

					//example
					//TSTR_CPY(args->argv[1], isy_T("test"));
					//TSTR_CPY(args->argv[2], isy_T("test"));

					return ISCRTNCB_Done;
				}
				break;
			} //end switch
		} //end ISCEVTTYPE_Query
		break;				
		
        case ISCEVTTYPE_Info:			
            switch (event->code) {				
               	case ISCEVT_InfSubsSetStarted:
               		TSTR_CPY(buf, ISYNCSTR_SubsSet);
               		TSTR_CAT(buf, event->state.currSubsSet);
					dataPtr->setStatus(buf, dataPtr->engine);
					appLogWrite(appPtr, event->code, event->type, buf);
					break;
				case ISCEVT_InfSyncingSubs:
					if (*event->state.currSubs)
					{
						TSTR_CPY(buf, ISYNCSTR_Subscription);
	               		TSTR_CAT(buf, event->state.currSubs);
						dataPtr->setStatus(buf, dataPtr->engine);
						appLogWrite(appPtr, event->code, event->type, buf);
					}
					break;

				case ISCEVT_InfSyncStarted:
					dataPtr->setStatus(ISYNCSTR_TryConnect, dataPtr->engine);
					appLogWrite(appPtr, event->code, event->type, statusMsg);
                    break;
					
				case ISCEVT_InfSyncSucceeded:
				case ISCEVT_InfSyncFailed:
				case ISCEVT_InfSyncCanceled:
					dataPtr->setStatus(statusMsg, dataPtr->engine);
					appLogWrite(appPtr, event->code, event->type, statusMsg);
                    break;
                    
                default: // ignore other event code
             	    if(TSTR_LEN(statusMsg) > 0)
					{	

						TSTR_CPY(buf, statusMsg);

						TSTR_CAT(buf, isy_T(", "));
						TSTR_CAT(buf, TSTR_ITOA(percent, event->state.syncProg));
						TSTR_CAT(buf, isy_T("%"));

						dataPtr->setStatus(buf, dataPtr->engine);
						appLogWrite(appPtr, event->code, event->type, statusMsg);
					}
                    break;
            } // switch (event->code)
            break;
        default:
        	if(TSTR_LEN(statusMsg) > 0)
			{	
				//appMessageBox(statusMsg, APPMB_OK);
				dataPtr->messageBox(statusMsg, dataPtr->engine, APPMB_OK, NULL);
				appLogWrite(appPtr, event->code, event->type, statusMsg);
			}			
            break;
    } // switch (event->type)

	return ISCRTNCB_Default;
}
