/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  Isync.cpp
 *
 * Description:
 *    Main isync application code
 *
 */

#include "db2sync.h"

// iSync.
#include "Dialogs.h"
#include "Model.h"
#include "logfile.h"

#include <e32base.h>
#include <s32file.h>
#include <e32cons.h>
#include <f32file.h>
#include <eikcmbut.h>
#include <sys/reent.h>
#include <estlib.h>

#define kButtonId_Synchronize	0

EXPORT_C CApaApplication* NewApplication()
{
	return new CISyncApplication;
}

GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}

//======================================================================
// CISyncApplication
TUid CISyncApplication::AppDllUid() const
{
	return KUidIkISync;
}

CApaDocument* CISyncApplication::CreateDocumentL()
{
	return CISyncDocument::NewL(*this);
	//return new (ELeave) CISyncDocument(*this);
}


//======================================================================
// CISyncDocument
CISyncDocument::CISyncDocument(CEikApplication& aApp)
	:CEikDocument(aApp)
{
}

// The document requires two-phase construction because it
// owns the model and therefore has to allocate 
// memory for it.
CISyncDocument* CISyncDocument::NewL(CEikApplication& aApp)
{
	CISyncDocument* self=new (ELeave) CISyncDocument(aApp);
	CleanupStack::PushL(self);
	self->ConstructL();
	self->ResetModelL();
	CleanupStack::Pop();
	return self;
}

void CISyncDocument::ConstructL()
{

	RFs fsSession;
    TInt rc;
	User::LeaveIfError(fsSession.Connect()); // Connect session
	_LIT(KAppDirName, "C:\\System\\Apps\\ISync\\");
	rc = fsSession.MkDir(KAppDirName);
	_LIT(KDataDirName, "C:\\System\\Data\\ISync\\");
	rc = fsSession.MkDir(KDataDirName);
	fsSession.Close();

	//create the globals
	epocCreateGlobals();

	//init the APPDATA struct
	appInitialize();

	//close the application log
	EPOCGLOBALS* globals = epocGetGlobals();
	APPDATA* appPtr = (APPDATA*) globals->hApp;
	logClose(appPtr->hLog);

	iModelPtr=new(ELeave) CModel;
	CleanupStack::PushL(iModelPtr);
	iModelPtr->ConstructL();
	CleanupStack::Pop();

}

// All resources allocated in ConstructL() must be released in 
// the destructor.
CISyncDocument::~CISyncDocument()
{

	delete iModelPtr;

	//terminate application
	//application log should already be closed
	//appTerminate();
	
	epocDestroyGlobals();
	CloseSTDLIB();
}

void CISyncDocument::ResetModelL()
{
}

CEikAppUi* CISyncDocument::CreateAppUiL()
{
    return new(ELeave) CISyncAppUi;
}

//======================================================================
// CISyncAppUi
void CISyncAppUi::ConstructL()
{
	BaseConstructL();

	CISyncDocument* docPtr=static_cast<CISyncDocument*>(iDocument);

	iModelPtr=docPtr->Model();

	iAppViewPtr=new(ELeave) CISyncAppView(this);
	iAppViewPtr->ConstructL(ClientRect());
}

CISyncAppUi::~CISyncAppUi()
{
	delete iAppViewPtr;
}

void CISyncAppUi::HandleCommandL(TInt aCommand)
{
	switch(aCommand)
	{
		case EApplicationsCmd:			ShowApplicationsDialog();		break;
		case EServerSettingsCmd:		ShowServerSettingsDialog();		break;
		case EClientSettingsCmd:		ShowClientSettingsDialog();		break;
		case ENetworkSettingsCmd:		ShowNetworkSettingsDialog();	break;
		case EShowLogCmd:				ShowLogDialog();				break;
		case EEikCmdExit:				Exit();							break;
		case ESynchronizeOptionsCmd:	ShowProgressDialog();			break;
		case EHelpCmd:					ShowAboutDialog();				break;
	}
}


void CISyncAppUi::ShowApplicationsDialog()
{
	CEikDialog* dialog=new(ELeave) CJobsDialog(iModelPtr);
	dialog->ExecuteLD(R_JOBS_DIALOG);
}


void CISyncAppUi::ShowServerSettingsDialog()
{
	CEikDialog* dialog=new (ELeave) CServerSettingsDialog();
	dialog->ExecuteLD(R_SETTINGS_DIALOG);
}

void CISyncAppUi::ShowClientSettingsDialog()
{
	CEikDialog* dialog=new (ELeave) CClientSettingsDialog();
	dialog->ExecuteLD(R_CLIENT_SETTINGS_DIALOG);
}

void CISyncAppUi::ShowNetworkSettingsDialog()
{
	CEikDialog* dialog=new (ELeave) CNetworkSettingsDialog();
	dialog->ExecuteLD(R_NETWORK_SETTINGS_DIALOG);
}

void CISyncAppUi::ShowJobsDialog()
{
	CEikDialog* dialog=new(ELeave) CJobsDialog(iModelPtr);
	dialog->ExecuteLD(R_JOBS_DIALOG);
}

void CISyncAppUi::ShowProgressDialog()
{
	CEikDialog* dialog=new(ELeave) CProgressDialog();
	dialog->ExecuteLD(R_PROGRESS_DIALOG);
}

void CISyncAppUi::ShowLogDialog()
{
	CEikDialog* dialog=new(ELeave) CLogDialog();
	dialog->ExecuteLD(R_LOG_DIALOG);
}

void CISyncAppUi::ShowMessageBoxDialog(isy_TCHAR* msg)
{
	CEikDialog* dialog=new(ELeave) CMessageBoxDialog(msg);
	dialog->ExecuteLD(R_MESSAGEBOX_CONT_DIALOG);
}

void CISyncAppUi::ShowAboutDialog()
{
	CEikDialog* dialog = new(ELeave) CAboutDialog();
	dialog->ExecuteLD(R_ABOUT_DIALOG);
}

//======================================================================
// CISyncAppView
CISyncAppView::CISyncAppView(CISyncAppUi* inUiPtr)
{
	iUiPtr=inUiPtr;
}

CISyncAppView::~CISyncAppView()
{
	delete iButtonsPtr;
}

void CISyncAppView::ConstructL(const TRect& aRect)
{
	CreateWindowL();
	SetRect(aRect);
	ActivateL();

	// read button names from resourcefile
	TBuf<128> buttonTitle_Synchronize;
	CEikonEnv::Static()->ReadResource(buttonTitle_Synchronize, R_ISYNC_BUTTONNAME_SYNCHRONIZE);

	ActivateL();
	DrawNow();
}

void CISyncAppView::AddButtonL(const TInt aId, const TDesC& aText)
{
	CEikCommandButton* button = new(ELeave) CEikCommandButton;
	CleanupStack::PushL(button);

	button->SetContainerWindowL(*this);
	if (aText.Length())
		button->SetTextL(aText);

	button->SetObserver(this);
	iButtonsPtr->AddControlL(button, aId);

	CleanupStack::Pop();
};

void CISyncAppView::Draw(const TRect& /*aRect*/)const
{

	CWindowGc& gc = SystemGc();
	// Clear the application view
	gc.Clear();

	// white out whole rectangle
	TRect rect=Rect();
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
	gc.SetBrushColor(KRgbWhite);
	gc.DrawRect(rect);
	gc.SetBrushStyle(CGraphicsContext::ENullBrush);	

	// Draw a rectangle round the edge of the application view.
	TRect drawRect=Rect();
	drawRect.Shrink(0,0);
	gc.SetPenSize(TSize(2,2));
	gc.DrawRect(drawRect);
}


void CISyncAppView::HandleControlEventL(CCoeControl* aControl, TCoeEvent /*aEventType*/)
{
	switch (iButtonsPtr->ControlId(aControl))
	{
	case kButtonId_Synchronize:
		iUiPtr->ShowProgressDialog();
		break;
	}
}

TInt CISyncAppView::CountComponentControls() const
{
	return (iButtonsPtr? iButtonsPtr->CountComponentControls() : 0);
}

CCoeControl* CISyncAppView::ComponentControl(TInt aIndex) const
{
	if(aIndex<iButtonsPtr->CountComponentControls())
	{
		return iButtonsPtr->ControlById(aIndex);
	}
	else return NULL;
}
