/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  osdefs.h
 *
 * Description:
 *    Definitions for OS-specific routines
 *
 **/

#ifndef _OSDEFS_EP_H_
#define _OSDEFS_EP_H_


#if defined(UNDER_EP) && defined(_UNICODE)
#include <ctype.h>
#include <stat.h>
#endif

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "isyncore.h"

#ifdef __cplusplus
extern "C" {
#endif

// C has no built-in true or false
#define false 0
#define true 1

long max(long num1, long num2);
long min(long num1, long num2);

//EPOC's directory for ISync files
#define ISYNCDIR isy_T("C:\\System\\Apps\\iSync\\")
#define CONFDIR ISYNCDIR
#define DATADIR isy_T("C:\\System\\Data\\iSync\\")

#define OSGET_CURRENT_DATE_TIME(y, m, d, hr, mi, se) epocGetCurrentDateTime(y, m, d, hr, mi, se)

// Debug
#define DECLARE_ERR_
#if defined(UNDER_EP) && defined(_UNICODE)
#  define OSS_ASSERT(expr, msg) EPOC_ASSERT_V6((int) expr, (isy_TCHAR*) msg)
void  EPOC_ASSERT_V6(int expr, isy_TCHAR* msg);
void EPOC_ASSERT(int expr, char* msg);
#else
#  define OSS_ASSERT(expr, msg) EPOC_ASSERT((int) expr, (char*) msg)
void EPOC_ASSERT(int expr, char* msg);
#endif

// Memory management
#  define MEM_HANDLE_ALLOC(size) malloc((unsigned long) (size))
#  define MEM_HANDLE_RESIZE(handle, size)	realloc((void *) handle, size)
#  define MEM_HANDLE_FREE(handle) free((void*) (handle))
#  define MEM_HANDLE_LOCK(handle) handle
#  define MEM_HANDLE_UNLOCK(handle)
#  define MEM_PTR_ALLOC(size) malloc((unsigned long) (size))
#  define MEM_PTR_FREE(ptr) free((void*) (ptr))
#  define MEM_SET(ptr, val, cb) memset((void*) (ptr), val, (unsigned long) (cb))
#  define MEM_CPY(pTo, pFrom, cb) memcpy((void*) (pTo), (void*) (pFrom), (unsigned long) (cb))

#define MEM_HEAP_FREE_BYTES(id, free, max) epocMemHeapFreeBytes(free, max);

// String manipulation
#if defined(UNDER_EP) && defined(_UNICODE)

#define TSTR_LEN(psz) wcslen((const wchar_t*) (psz))
#define TSTR_CPY(pDst, pSrc) wcscpy((wchar_t*) (pDst), (const wchar_t*) (pSrc))
#define TSTR_CAT(pDst, pSrc) wcscat((wchar_t*) (pDst), (const wchar_t*) (pSrc))
#define TSTR_CMP(pDst, pSrc) wcscmp((const wchar_t*)(pDst), (const wchar_t*) (pSrc))

#define TSTR_CHR(pStr, chr) wcschr((const wchar_t*) (pStr), (wchar_t) (chr))
#define TSTR_R_CHR(pStr, chr) wcsrchr((const wchar_t*) (pStr), (wchar_t) (chr))
#define TSTR_STR(pStr, pStr2) wcsstr((const wchar_t*) pStr, (const wchar_t*) pStr2)
#define TSTR_N_CPY(pDst, pSrc, cb) wcsncpy((wchar_t*) (pDst), (const wchar_t*) (pSrc), cb)
#define TSTR_N_CAT(pDst, pSrc, cb) wcsncat((wchar_t*) (pDst), (const wchar_t*) (pSrc), cb)
#define TSTR_I_CMP(pDst, pSrc) wcsicmp((const wchar_t*) (pDst), (const wchar_t*) (pSrc))
#define TSTR_NI_CMP(pDst, pSrc, cb) wcsnicmp((const wchar_t*) (pDst), (const wchar_t*) (pSrc), cb)
#define TSTR_ITOA(buf, i) IntToWstr((wchar_t*) buf, i)
#define TSTR_ATOI(buf) WstrToInt((const wchar_t*) buf)
#define TSTR_ISDIGIT(c) (((isy_TCHAR) '0' <= c) && (c <= (isy_TCHAR) '9'))
#define towlower(c) (((c) >= (isy_TCHAR) 'A' && (c) <= (isy_TCHAR) 'Z') ? ((c) - (isy_TCHAR) 'A' + (isy_TCHAR) 'a') : (c))
#define towupper(c) (((c) >= (isy_TCHAR) 'a' && (c) <= (isy_TCHAR) 'z') ? ((c) - (isy_TCHAR) 'a' + (isy_TCHAR) 'A') : (c))

wchar_t* wcschr(const wchar_t* pStr, wchar_t chr);
wchar_t* wcsrchr(const wchar_t* pStr, wchar_t chr);
wchar_t* wcsstr(const wchar_t* pStr, const wchar_t* pStr2);
wchar_t* wcsncpy(wchar_t* pDst, const wchar_t* pSrc, long cb);
wchar_t* wcsncat(wchar_t* pDst, const wchar_t* pSrc, long cb);
int wcsicmp(const wchar_t* pDst, const wchar_t* pSrc);
int wcsnicmp (const wchar_t* pDst, const wchar_t* pSrc, long cb);
wchar_t* IntToWstr(wchar_t* s, int n);
int WstrToInt(const wchar_t* s);

#else
#  define TSTR_LEN	STR_LEN
#  define TSTR_CPY    STR_CPY
#  define TSTR_CAT    STR_CAT
#  define TSTR_CMP    STR_CMP
#  define TSTR_CHR    STR_CHR
#  define TSTR_STR    STR_STR
#  define TSTR_N_CPY  STR_N_CPY
#  define TSTR_N_CAT  STR_N_CAT
#  define TSTR_I_CMP  STR_I_CMP
#  define TSTR_NI_CMP STR_NI_CMP
#  define TSTR_N_CMP  STR_N_CMP
#  define TSTR_ITOA   STR_ITOA
#  define TSTR_ATOI   STR_ATOI
#endif

#define STR_LEN(psz) strlen((char*) (psz))
#define STR_CPY(pDst, pSrc) strcpy(pDst, pSrc)
#define STR_CAT(pDst, pSrc) strcat((char*) (pDst), (char*) (pSrc))
#define STR_N_CPY(pDst, pSrc, cb) strncpy((char*) (pDst), (char*) (pSrc), cb)
#define STR_N_CAT(pDst, pSrc, cb) strncat((char*) (pDst), (char*) (pSrc), cb)
#define STR_NI_CMP(pDst, pSrc, cb) strncasecmp((char*) (pDst), (char*) (pSrc), cb)
#define STR_I_CMP(pDst, pSrc) strcasecmp((char*) (pDst), (char*) (pSrc))
#define STR_CMP(pDst, pSrc) strcmp((char*) (pDst), (char*) (pSrc))
#define STR_ITOA(buf, i) itoa(buf, i)
#define STR_ATOI(buf) atoi(buf)
#define STR_CHR(pStr, chr) strchr((char*) (pStr), (int) (chr))
#define STR_STR(pStr, pStr2) strstr((char*) pStr, (char*) pStr2)
#define STR_ISDIGIT(c) isdigit(c)

char* itoa(char s[], int n);
char* TcharToChar(isy_TCHAR* tchar_str, char* char_str);

// File system
#define FSEEK_BEGINNING SEEK_SET
#define FSEEK_CURRENT SEEK_CUR
#define FSEEK_END SEEK_END
#define FILE_NL isy_T("\r\n")
#define FPATH_SLASH		'\\'
#define FPATH_SLASHSTR	"\\"

#define HISYNCFILE  FILE*

#if defined(UNDER_EP) && defined(_UNICODE)
#  define FOPEN(fileName, openMode) wfopen((wchar_t*)fileName, (wchar_t*)openMode)
#  define REMOVE(fileName) wremove(fileName)
#  define FMKDIR(dir) wmkdir((wchar_t*)dir, 0700)
#else
#  define FOPEN(fileName, openMode) fopen(fileName, openMode)
#  define REMOVE(fileName) remove(fileName)
#  define FMKDIR(dir) mkdir(dir, 0700)
#endif
#define FCLOSE(file) fclose(file)
#define FREAD(array, objSize, numObj, file) fread(array, objSize, numObj, file)
#define FWRITE(array, objSize, numObj, file) fwrite(array, objSize, numObj, file)
#define FSEEK(file, offset, origin) fseek(file, offset, origin)
#define REWIND(file) rewind(file)
#define FFLUSH(file) fflush(file)

#ifdef __cplusplus
}
#endif

#include "Epocosdefs.h"

#endif //_OSDEFS_EP_H_

