/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  logDisplay.cpp
 *
 * Description:
 *    Displays application log
 *
 */

#include "stdafx.h"
#include "logDisplay.h"
#include "db2sync.h"

/**********************************************************************************
 Message handler for log box.
 *********************************************************************************/
LRESULT CALLBACK LogDisplay(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int NewPosX = 0, NewPosY = 0;
	
	switch (message)
	{
		case WM_INITDIALOG:
		{
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_LOG), ISYNCSTR_Log);
			SetWindowText(GetDlgItem (hDlg, IDOK), ISYNCSTR_OK);

			GET_APP_
			FILE *logFile = NULL;
			isy_TCHAR logFileName[128];
			TSTR_CPY(logFileName, DATADIR);
			//TSTR_CPY(logFileName, appPrefTargetDir(_pApp));
			TSTR_CAT(logFileName, appLogDbName);

			logFile = FOPEN(logFileName, isy_T("rb"));
			if ( logFile != NULL ) {
				SetLogTextArea(hDlg, IDC_LOG_TEXT_AREA, logFile);	
				FCLOSE(logFile);
			}
			return centerHandle(hDlg, NewPosX, NewPosY, 0 , 0);
		}
		case WM_COMMAND:
			if ((LOWORD(wParam) == IDOK) || (LOWORD(wParam) == IDCANCEL))
			{
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}

/********************************************************************
 SetLogTextArea
 Reads the log filehandle and sets IDC_LOG_TEXT_AREA text with
 the logfile contents.
 ********************************************************************/
isy_BOOL SetLogTextArea(HWND hDlg, isy_INT32 CONTROLID, FILE* logFile) {

	long logFileSize = 0;
	isy_TCHAR * tbuf = NULL;

	logFileSize = getFileSize(logFile);
	tbuf = (isy_TCHAR *) MEM_HANDLE_ALLOC(logFileSize+sizeof(isy_TCHAR));
	if (tbuf)
		MEM_SET(tbuf, 0, logFileSize+sizeof(isy_TCHAR));
	if (tbuf && logFileSize != 0) 
	{
		FREAD(tbuf, sizeof(isy_TCHAR), logFileSize/sizeof(isy_TCHAR), logFile);
		//strip BOM for UTF-16LE
		if ((isy_BYTE) *tbuf == 0xFF && (isy_BYTE) ((*tbuf >> 8) & 0xFF) == 0xFE)
			SetDlgItemText(hDlg, CONTROLID, tbuf+1);
		else
			SetDlgItemText(hDlg, CONTROLID, tbuf);
		MEM_HANDLE_FREE(tbuf);
		return true;
	}
	else 
	{
		return false;
	}

}

/********************************************************************
 getFileSize (in bytes)
 Side effects: file original position resets to zero
 ********************************************************************/
isy_UINT32 getFileSize( FILE* file )
{
	long size = 0;

	FSEEK(file, 0, FSEEK_END);
	size=ftell(file);
    FSEEK(file, 0, FSEEK_BEGINNING);
	return size;
}