/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  osdefs_ce.c
 *
 * Description:
 *    Common source file for OS-specific definitions
 */

#include "osdefs.h"

//#include <assert.h>

// Timestamp functions
void ceGetCurrentDateTime(int* py, int* pm, int* pd, int* phr, int* pmi, int *pse)
{
	FILETIME fileTime, localFileTime;
	SYSTEMTIME utcTime, localTime;

	GetSystemTime(&utcTime);
	SystemTimeToFileTime(&utcTime,&fileTime);
	FileTimeToLocalFileTime(&fileTime, &localFileTime);
	FileTimeToSystemTime(&localFileTime, &localTime); 
	*py = localTime.wYear;
	*pm = localTime.wMonth;
	*pd = localTime.wDay;
	*phr = localTime.wHour;
	*pmi = (int)localTime.wMinute;
	*pse = (int)localTime.wSecond;

}

void WINCE_ASSERT(int expr, TCHAR * msg)
{
	if (!expr) {

		//TODO: wrap the C++ panic
		wprintf(msg);
		wprintf(L"\n");
		//exit(-1);
		//assert(-1);
	}
}


// WinCE passes the number of bytes of a file was read by reference parameter.
// This wrapper returns it instead. Takes same types as standard fread.
size_t WINCE_fread(void* lpBuffer, size_t nNumberOfBytesToRead, size_t lpNumberOfBytesRead, HANDLE hFile)
{
	ReadFile(hFile, lpBuffer, nNumberOfBytesToRead, &lpNumberOfBytesRead, NULL);
	return lpNumberOfBytesRead;
}


// WinCE passes the number of bytes of a file was written by reference parameter.
// This wrapper returns it instead. Takes same types as standard fwrite.
size_t WINCE_fwrite(void* lpBuffer, size_t nNumberOfBytesToWrite, size_t lpNumberOfBytesWritten, HANDLE hFile)
{
	WriteFile(hFile, lpBuffer, nNumberOfBytesToWrite, &lpNumberOfBytesWritten, NULL);
	return lpNumberOfBytesWritten;
}


void MemHeapFreeBytes(int id, unsigned long* free, unsigned long* max)
{
	MEMORYSTATUS mems;
	GlobalMemoryStatus(&mems);
	*free = mems.dwAvailVirtual; // ??
	*max = mems.dwTotalVirtual; // ??
}

// _strnicmp not supported in CE
long CE_strnicmp(char *s1, char *s2, long n)
{
	if (n == 0)
		return 0;
 
	while (n-- != 0 && tolower(*s1) == tolower(*s2))
	{
		if (n == 0 || *s1 == '\0' || *s2 == '\0')
			break;
		s1++;
		s2++;
	}

	return tolower(*(unsigned char *) s1) - tolower(*(unsigned char *) s2);
}


// _stricmp not supported in CE
long CE_stricmp(char* str1, char* str2) {
	unsigned long str_len1,str_len2,len,i;

	str_len1=strlen(str1);
	str_len2=strlen(str2);
    len=(str_len1>str_len2)? str_len2:str_len1;
	for( i = 0; i < len; i++ )
	{
		if (tolower(str1[i]) < tolower(str2[i]))
			return -1;
		else if (tolower(str1[i]) > tolower(str2[i]))
			return 1;
	}
    if (str_len1==str_len2)
		return 0;
	else if (str_len1<str_len2)
	    return -1;
	else return 1;
}
